@@ -176,20 +176,23 @@ gst_alsa_class_init (gpointer g_class, gpointer class_data)
           "Name of the device", NULL, G_PARAM_READABLE));
   g_object_class_install_property (object_class, ARG_PERIODCOUNT,
       g_param_spec_int ("period-count", "Period count",
-          "Number of hardware buffers to use",
-          2, 64, 2, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+          "Number of hardware buffers to use (0 for card's default)",
+          GST_ALSA_MIN_PERIOD_CNT, GST_ALSA_MAX_PERIOD_CNT,
+          GST_ALSA_MIN_PERIOD_CNT, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
   g_object_class_install_property (object_class, ARG_PERIODSIZE,
       g_param_spec_int ("period-size", "Period size",
           "Number of frames (samples on each channel) in one hardware period",
-          2, 8192, 8192, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+          GST_ALSA_MIN_PERIOD_SZ, GST_ALSA_MAX_PERIOD_SZ,
+          GST_ALSA_MAX_PERIOD_SZ, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
   g_object_class_install_property (object_class, ARG_BUFFERSIZE,
       g_param_spec_int ("buffer-size", "Buffer size",
           "Number of frames the hardware buffer can hold",
-          4, 65536, 16384, G_PARAM_READWRITE));
+          GST_ALSA_MIN_BUFFER_SZ, GST_ALSA_MAX_BUFFER_SZ,
+          GST_ALSA_MIN_PERIOD_CNT * GST_ALSA_MAX_PERIOD_SZ, G_PARAM_READWRITE));
   g_object_class_install_property (object_class, ARG_AUTORECOVER,
       g_param_spec_boolean ("autorecover", "Automatic xrun recovery",
-          "When TRUE tries to reduce processor load on xruns",
-          TRUE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+          "When TRUE tries to reduce processor load on xruns", TRUE,
+          G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
   g_object_class_install_property (object_class, ARG_MMAP,
       g_param_spec_boolean ("mmap", "Use mmap'ed access",
           "Wether to use mmap (faster) or standard read/write (more compatible)",
@@ -211,6 +214,8 @@ static void
 gst_alsa_init (GstAlsa * this)
 {
   this->device = g_strdup ("default");
+  this->cached_caps = NULL;
+  this->period_count = 2;
 
   GST_FLAG_SET (this, GST_ELEMENT_EVENT_AWARE);
   GST_FLAG_SET (this, GST_ELEMENT_THREAD_SUGGESTED);
@@ -254,7 +259,7 @@ gst_alsa_set_property (GObject * object, guint prop_id, const GValue * value,
       break;
     case ARG_PERIODCOUNT:
       g_return_if_fail (!GST_FLAG_IS_SET (this, GST_ALSA_RUNNING));
-      this->period_count = g_value_get_int (value);
+      this->desired_period_count = g_value_get_int (value);
       break;
     case ARG_PERIODSIZE:
       g_return_if_fail (!GST_FLAG_IS_SET (this, GST_ALSA_RUNNING));
@@ -263,7 +268,7 @@ gst_alsa_set_property (GObject * object, guint prop_id, const GValue * value,
     case ARG_BUFFERSIZE:
       g_return_if_fail (!GST_FLAG_IS_SET (this, GST_ALSA_RUNNING));
       buffer_size = g_value_get_int (value);
-      this->period_count = buffer_size / this->period_size;
+      this->desired_period_count = buffer_size / this->period_size;
       break;
     case ARG_AUTORECOVER:
       this->autorecover = g_value_get_boolean (value);
@@ -579,6 +584,7 @@ gst_alsa_get_format (const GstStructure * structure)
 {
   const gchar *mimetype;
   GstAlsaFormat *ret;
+  gint rate, chans;
 
   if (!(ret = g_new (GstAlsaFormat, 1)))
     return NULL;
@@ -642,10 +648,13 @@ gst_alsa_get_format (const GstStructure * structure)
   }
 
   /* get rate and channels */
-  if (!(gst_structure_get_int (structure, "rate", &ret->rate) &&
-          gst_structure_get_int (structure, "channels", &ret->channels)))
+  if (!gst_structure_get_int (structure, "rate", &rate)
+      || !gst_structure_get_int (structure, "channels", &chans))
     goto error;
 
+  ret->rate = rate;
+  ret->channels = chans;
+
   return ret;
 
 error:
@@ -715,8 +724,7 @@ gst_alsa_get_caps_internal (snd_pcm_format_t format)
 }
 
 static inline void
-add_channels (GstStructure * structure, gint min_rate, gint max_rate,
-    gint min_channels, gint max_channels)
+add_rates (GstStructure * structure, gint min_rate, gint max_rate)
 {
   if (min_rate < 0) {
     min_rate = GST_ALSA_MIN_RATE;
@@ -739,6 +747,11 @@ add_channels (GstStructure * structure, gint min_rate, gint max_rate,
     gst_structure_set (structure, "rate", GST_TYPE_INT_RANGE, min_rate,
         max_rate, NULL);
   }
+}
+
+static inline void
+add_channels (GstStructure * structure, gint min_channels, gint max_channels)
+{
   if (min_channels < 0) {
     min_channels = 1;
     max_channels = GST_ALSA_MAX_CHANNELS;
@@ -781,7 +794,8 @@ gst_alsa_caps (snd_pcm_format_t format, gint rate, gint channels)
     g_assert (ret_caps != NULL);
     g_assert (gst_caps_get_size (ret_caps) == 1);
 
-    add_channels (gst_caps_get_structure (ret_caps, 0), rate, -1, channels, -1);
+    add_rates (gst_caps_get_structure (ret_caps, 0), rate, -1);
+    add_channels (gst_caps_get_structure (ret_caps, 0), channels, -1);
   } else {
     int i;
     GstCaps *temp;
@@ -793,7 +807,8 @@ gst_alsa_caps (snd_pcm_format_t format, gint rate, gint channels)
       /* can be NULL, because not all alsa formats can be specified as caps */
       if (temp != NULL) {
         g_assert (gst_caps_get_size (temp) == 1);
-        add_channels (gst_caps_get_structure (temp, 0), rate, -1, channels, -1);
+        add_rates (gst_caps_get_structure (temp, 0), rate, -1);
+        add_channels (gst_caps_get_structure (temp, 0), channels, -1);
         gst_caps_append (ret_caps, temp);
       }
     }
@@ -803,6 +818,122 @@ gst_alsa_caps (snd_pcm_format_t format, gint rate, gint channels)
   return ret_caps;
 }
 
+
+static int
+gst_alsa_check_sample_rates (snd_pcm_t * device_handle,
+    snd_pcm_hw_params_t * hw_params, unsigned int *tested_rates,
+    GValue * supported_rates)
+{
+  int i;
+  char init_done = 0;
+
+  GValue value = { 0 };
+  g_value_init (&value, G_TYPE_INT);
+
+  for (i = 0; tested_rates[i] != 0; i++) {
+    if (!snd_pcm_hw_params_test_rate (device_handle, hw_params, tested_rates[i],
+            0)) {
+      if (!init_done) {
+        /* at least one sample rate supported */
+        g_value_init (supported_rates, GST_TYPE_LIST);
+        init_done = 1;
+      }
+
+      g_value_set_int (&value, tested_rates[i]);
+      gst_value_list_append_value (supported_rates, &value);
+    }
+  }
+
+// only one -> G_TYPE_INT
+
+  g_value_unset (&value);
+
+  return init_done;
+}
+
+static int
+gst_alsa_rates_probe (snd_pcm_t * device_handle,
+    snd_pcm_hw_params_t * hw_params, GValue * supported_rates)
+{
+  int n;
+  gboolean min_found = FALSE, max_found = FALSE;
+  unsigned int common_rates[] =
+      { 8000, 11025, 12000, 16000, 22050, 24000, 32000, 44100, 48000, 88200,
+    96000, 192000, 0, 0, 0
+  };
+  /* this dummy sample rate should only be supported by a software device.
+   * you need two, because one could be dmix... */
+  unsigned int uncommon_rates[] = { 12345, 45678, 0 };
+
+  int ret;
+  int dir;
+  unsigned int min_rate, max_rate;
+
+  /* Get MIN/MAX supported rate */
+  snd_pcm_hw_params_get_rate_min (hw_params, &min_rate, &dir);
+  min_rate =
+      min_rate <
+      GST_ALSA_MIN_RATE ? GST_ALSA_MIN_RATE : (min_rate +
+      GST_ALSA_DIR_MIN (dir));
+  snd_pcm_hw_params_get_rate_max (hw_params, &max_rate, &dir);
+  max_rate =
+      max_rate >
+      GST_ALSA_MAX_RATE ? GST_ALSA_MAX_RATE : (max_rate +
+      GST_ALSA_DIR_MAX (dir));
+  for (n = 0; common_rates[n] != 0; n++) {
+    if (common_rates[n] == min_rate)
+      min_found = TRUE;
+    if (common_rates[n] == max_rate)
+      max_found = TRUE;
+  }
+  if (!min_found)
+    common_rates[12] = min_rate;
+  if (!max_found && min_rate != max_rate)
+    common_rates[13] = max_rate;
+
+  ret =
+      gst_alsa_check_sample_rates (device_handle, hw_params, uncommon_rates,
+      supported_rates);
+  if (ret) {
+    /* Uncommon sample rates supported, it is certainly a
+     * "software"/dummy device */
+    g_value_unset (supported_rates);
+    g_value_init (supported_rates, GST_TYPE_INT_RANGE);
+
+    if (min_rate != max_rate) {
+      gst_value_set_int_range (supported_rates, min_rate, max_rate);
+    } else {
+      /* Only one supported */
+      g_value_unset (supported_rates);
+      g_value_init (supported_rates, G_TYPE_INT);
+      g_value_set_int (supported_rates, min_rate);
+    }
+  } else {
+    /* Check common sample rates for real hardware support */
+    ret =
+        gst_alsa_check_sample_rates (device_handle, hw_params, common_rates,
+        supported_rates);
+    if (ret) {
+      if (gst_value_list_get_size (supported_rates) == 1) {
+        /* Only one supported */
+        GValue *rate = (GValue *) gst_value_list_get_value (supported_rates, 0);
+
+        min_rate = g_value_get_int (rate);
+        g_value_unset (supported_rates);
+        g_value_init (supported_rates, G_TYPE_INT);
+        g_value_set_int (supported_rates, min_rate);
+      }
+    } else {
+      GST_WARNING ("No supported samplerates found for %d-%d range",
+          min_rate, max_rate);
+      gst_value_set_int_range (supported_rates, min_rate, max_rate);
+    }
+  }
+
+  return 0;
+}
+
+
 /* Return better caps when device is open */
 GstCaps *
 gst_alsa_get_caps (GstPad * pad)
@@ -811,7 +942,11 @@ gst_alsa_get_caps (GstPad * pad)
   snd_pcm_hw_params_t *hw_params;
   snd_pcm_format_mask_t *mask;
   int i;
-  unsigned int min_rate, max_rate;
+  GValue supported_rates = { 0 };
+  unsigned int min_period_cnt, max_period_cnt;
+  snd_pcm_uframes_t min_period_sz, max_period_sz;
+  snd_pcm_uframes_t min_buffer_sz, max_buffer_sz;
+  gint buffer_size;
   gint min_channels, max_channels;
   GstCaps *ret = NULL;
 
@@ -832,21 +967,87 @@ gst_alsa_get_caps (GstPad * pad)
     min_channels = 1;
     max_channels = -1;
   } else {
-    ERROR_CHECK (snd_pcm_hw_params_get_channels_min (hw_params, &min_rate),
+    guint min, max;
+
+    ERROR_CHECK (snd_pcm_hw_params_get_channels_min (hw_params, &min),
         "Couldn't get minimum channel count for device %s: %s", this->device);
-    ERROR_CHECK (snd_pcm_hw_params_get_channels_max (hw_params, &max_rate),
+    ERROR_CHECK (snd_pcm_hw_params_get_channels_max (hw_params, &max),
         "Couldn't get maximum channel count for device %s: %s", this->device);
-    min_channels = min_rate;
-    max_channels =
-        max_rate > GST_ALSA_MAX_CHANNELS ? GST_ALSA_MAX_CHANNELS : max_rate;
+    min_channels = (min < 1) ? 1 : min;
+    max_channels = (max > GST_ALSA_MAX_CHANNELS) ? GST_ALSA_MAX_CHANNELS : max;
   }
 
-  ERROR_CHECK (snd_pcm_hw_params_get_rate_min (hw_params, &min_rate, &i),
-      "Couldn't get minimum rate for device %s: %s", this->device);
-  min_rate = min_rate < GST_ALSA_MIN_RATE ? GST_ALSA_MIN_RATE : min_rate + i;
-  ERROR_CHECK (snd_pcm_hw_params_get_rate_max (hw_params, &max_rate, &i),
-      "Couldn't get maximum rate for device %s: %s", this->device);
-  max_rate = max_rate > GST_ALSA_MAX_RATE ? GST_ALSA_MAX_RATE : max_rate + i;
+  /* Check available sample rates */
+  if (!gst_alsa_rates_probe (this->handle, hw_params, &supported_rates)) {
+    ;
+  }
+
+  /* Probe period_count and adjust default/user provided value against probed MIN/MAX */
+  ERROR_CHECK (snd_pcm_hw_params_get_periods_min (hw_params, &min_period_cnt,
+          &i), "Couldn't get minimum period_count for device %s: %s",
+      this->device);
+  min_period_cnt =
+      min_period_cnt <
+      GST_ALSA_MIN_PERIOD_CNT ? GST_ALSA_MIN_PERIOD_CNT : (min_period_cnt +
+      GST_ALSA_DIR_MIN (i));
+  if (this->period_count < min_period_cnt) {
+    this->period_count = min_period_cnt;
+  };
+  ERROR_CHECK (snd_pcm_hw_params_get_periods_max (hw_params, &max_period_cnt,
+          &i), "Couldn't get maximum period_count for device %s: %s",
+      this->device);
+  max_period_cnt =
+      max_period_cnt >
+      GST_ALSA_MAX_PERIOD_CNT ? GST_ALSA_MAX_PERIOD_CNT : (max_period_cnt +
+      GST_ALSA_DIR_MAX (i));
+  if (this->period_count > max_period_cnt) {
+    this->period_count = max_period_cnt;
+  };
+
+  /* Probe period_size and adjust default/user provided value against probed MIN/MAX */
+  ERROR_CHECK (snd_pcm_hw_params_get_period_size_min (hw_params, &min_period_sz,
+          &i), "Couldn't get minimum period_size for device %s: %s",
+      this->device);
+  min_period_sz =
+      min_period_sz <
+      GST_ALSA_MIN_PERIOD_SZ ? GST_ALSA_MIN_PERIOD_SZ : (min_period_sz +
+      GST_ALSA_DIR_MIN (i));
+  if (this->period_size < min_period_sz) {
+    this->period_size = min_period_sz;
+  };
+  ERROR_CHECK (snd_pcm_hw_params_get_period_size_max (hw_params, &max_period_sz,
+          &i), "Couldn't get maximum period_size for device %s: %s",
+      this->device);
+  max_period_sz =
+      max_period_sz >
+      GST_ALSA_MAX_PERIOD_SZ ? GST_ALSA_MAX_PERIOD_SZ : (max_period_sz +
+      GST_ALSA_DIR_MAX (i));
+  if (this->period_size > max_period_sz) {
+    this->period_size = max_period_sz;
+  };
+
+  /* Probe buffer_size MIN/MAX */
+  buffer_size = this->period_count * this->period_size;
+  ERROR_CHECK (snd_pcm_hw_params_get_buffer_size_min (hw_params,
+          &min_buffer_sz), "Couldn't get minimum buffer_size for device %s: %s",
+      this->device);
+  min_buffer_sz =
+      min_buffer_sz <
+      GST_ALSA_MIN_BUFFER_SZ ? GST_ALSA_MIN_BUFFER_SZ : min_buffer_sz;
+  if (buffer_size < min_buffer_sz) {
+    buffer_size = min_buffer_sz;
+    this->period_size = GST_ALSA_MIN_BUFFER_SZ / this->period_count;
+  };
+  ERROR_CHECK (snd_pcm_hw_params_get_buffer_size_max (hw_params,
+          &max_buffer_sz), "Couldn't get maximum buffer_size for device %s: %s",
+      this->device);
+  max_buffer_sz =
+      max_buffer_sz >
+      GST_ALSA_MAX_BUFFER_SZ ? GST_ALSA_MAX_BUFFER_SZ : max_buffer_sz;
+  if (buffer_size > max_buffer_sz) {
+    buffer_size = max_buffer_sz;
+    this->period_size = GST_ALSA_MAX_BUFFER_SZ / this->period_count;
+  };
 
   snd_pcm_format_mask_alloca (&mask);
   snd_pcm_hw_params_get_format_mask (hw_params, mask);
@@ -859,8 +1060,12 @@ gst_alsa_get_caps (GstPad * pad)
         gint n;
 
         g_assert (gst_caps_get_size (caps) == 1);
-        add_channels (gst_caps_get_structure (caps, 0), min_rate, max_rate,
-            min_channels, max_channels);
+
+        gst_structure_set_value (gst_caps_get_structure (caps, 0), "rate",
+            &supported_rates);
+
+        add_channels (gst_caps_get_structure (caps, 0), min_channels,
+            max_channels);
 
         /* channel configuration */
         /* MIN used to spped up because we don't support more than 8 channels */
@@ -919,6 +1124,8 @@ gst_alsa_get_caps (GstPad * pad)
   gst_caps_do_simplify (ret);
   GST_LOG_OBJECT (this, "get_caps returns %P", ret);
 
+  g_value_unset (&supported_rates);
+
   this->cached_caps = gst_caps_copy (ret);
   return ret;
 }
@@ -1046,6 +1253,7 @@ gst_alsa_link (GstPad * pad, const GstCaps * caps)
   GstAlsa *this;
   GstAlsaFormat *format;
   GstPadLinkReturn ret;
+  gint old_rate = 0;
 
   g_return_val_if_fail (caps != NULL, GST_PAD_LINK_REFUSED);
   g_return_val_if_fail (pad != NULL, GST_PAD_LINK_REFUSED);
@@ -1110,8 +1318,12 @@ gst_alsa_link (GstPad * pad, const GstCaps * caps)
     /* sync the params */
     if (GST_FLAG_IS_SET (this, GST_ALSA_RUNNING))
       gst_alsa_stop_audio (this);
+    if (this->format)
+      old_rate = this->format->rate;
     g_free (this->format);
     this->format = format;
+    if (this->played && old_rate)
+      this->played = this->played * this->format->rate / old_rate;
     if (!gst_alsa_start_audio (this)) {
       GST_ELEMENT_ERROR (this, RESOURCE, SETTINGS, (NULL), (NULL));
       return GST_PAD_LINK_REFUSED;
@@ -1189,6 +1401,8 @@ gst_alsa_change_state (GstElement * element)
         gst_alsa_stop_audio (this);
       g_free (this->format);
       this->format = NULL;
+      this->played = 0;
+      this->captured = 0;
       break;
     case GST_STATE_READY_TO_NULL:
       if (GST_FLAG_IS_SET (element, GST_ALSA_OPEN))
@@ -1483,9 +1697,11 @@ gst_alsa_probe_hw_params (GstAlsa * this, GstAlsaFormat * format)
     GST_WARNING_OBJECT (this, "set rate (%u) differs from desired rate (%u)",
         rate, format->rate);
 
-  period_count = this->period_count;
-  SIMPLE_ERROR_CHECK (snd_pcm_hw_params_set_periods_near (this->handle,
-          hw_params, &period_count, 0));
+  if (this->desired_period_count > 0) {
+    period_count = this->desired_period_count;
+    SIMPLE_ERROR_CHECK (snd_pcm_hw_params_set_periods_near (this->handle,
+            hw_params, &period_count, 0));
+  }
   period_size = this->period_size;
   SIMPLE_ERROR_CHECK (snd_pcm_hw_params_set_period_size_near (this->handle,
           hw_params, &period_size, 0));
@@ -1503,6 +1719,8 @@ gst_alsa_set_hw_params (GstAlsa * this)
 {
   snd_pcm_hw_params_t *hw_params;
   unsigned int rate;
+  int ret;
+  snd_pcm_uframes_t period_size;
 
   g_return_val_if_fail (this != NULL, FALSE);
   g_return_val_if_fail (this->handle != NULL, FALSE);
@@ -1547,13 +1765,31 @@ gst_alsa_set_hw_params (GstAlsa * this)
     if (rate != this->format->rate)
       GST_WARNING_OBJECT (this, "set rate (%u) differs from desired rate (%u)",
           rate, this->format->rate);
-    ERROR_CHECK (snd_pcm_hw_params_set_periods_near (this->handle, hw_params,
-            &this->period_count, 0), "error setting period count to %u: %s",
-        (guint) this->period_count);
+    this->period_count =
+        this->desired_period_count ? this->desired_period_count : 2;
+    ret =
+        snd_pcm_hw_params_set_periods_near (this->handle, hw_params,
+        &this->period_count, 0);
+    if (ret < 0) {
+      /* if there's a specific period count requested, we have an error now. 
+       * Otherwise, we just don't care */
+      if (this->desired_period_count) {
+        GST_ERROR ("could not set period count to %u: %s",
+            (guint) this->period_count, snd_strerror (ret));
+        return FALSE;
+      } else {
+        GST_INFO ("could not set period count to %u: %s - using default",
+            (guint) this->period_count, snd_strerror (ret));
+      }
+    }
+
+    period_size = this->period_size;
     ERROR_CHECK (snd_pcm_hw_params_set_period_size_near (this->handle,
-            hw_params, &this->period_size, 0),
+            hw_params, &period_size, 0),
         "error setting period size to %u frames: %s",
-        (guint) this->period_size);
+        (guint) period_size);
+    if (this->period_size == 0)
+      this->period_size = period_size;
   } else {
     GST_INFO_OBJECT (this, "Preparing format: (none)");
   }
@@ -1563,6 +1799,8 @@ gst_alsa_set_hw_params (GstAlsa * this)
   ERROR_CHECK (snd_pcm_hw_params (this->handle, hw_params),
       "Could not set hardware parameters: %s");
 
+  ERROR_CHECK (snd_pcm_hw_params_get_periods (hw_params, &this->period_count,
+          0), "Could not get period count: %s");
   /* now get the pcm caps */
   GST_ALSA_CAPS_SET (this, GST_ALSA_CAPS_PAUSE,
       snd_pcm_hw_params_can_pause (hw_params));
@@ -1721,6 +1959,8 @@ gst_alsa_close_audio (GstAlsa * this)
     gst_caps_free (this->cached_caps);
     this->cached_caps = NULL;
   }
+  this->period_count =
+      this->desired_period_count ? this->desired_period_count : 2;
 
   return TRUE;
 }
