@@ -21,6 +21,8 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
+#include <string.h>
+#include <strings.h>
 #include <gst/gst.h>
 #include "gsttextoverlay.h"
 
@@ -84,6 +86,13 @@ static GstElementClass *parent_class = NULL;
 
 /*static guint gst_textoverlay_signals[LAST_SIGNAL] = { 0 }; */
 
+/* This is for cairo 0.4, remove once we require 0.5 */
+#ifndef CAIRO_API_SHAKEUP_FLAG_DAY
+#define cairo_set_source_rgba(cr,red,green,blue,alpha) {\
+    cairo_set_rgb_color(cr,red,green,blue); \
+    cairo_set_alpha(cr,alpha); }
+#endif
+
 
 GType
 gst_textoverlay_get_type (void)
@@ -213,38 +222,72 @@ gst_textoverlay_render_text (GstTextOverlay * overlay, gchar * text,
 {
   cairo_text_extents_t extents;
   char *string;
+  double x, y;
 
-  string = g_strdup_printf ("%.*s", textlen, text);
+  string = g_strndup (text, textlen);
 
-  if (overlay->pixbuf)
-    g_free (overlay->pixbuf);
-  overlay->pixbuf = g_malloc (4 * overlay->width * overlay->text_height);
-  cairo_set_target_image (overlay->cr, overlay->pixbuf, CAIRO_FORMAT_ARGB32,
-      overlay->width, overlay->text_height, overlay->width * 4);
+  if (overlay->text_fill_image)
+    g_free (overlay->text_fill_image);
+  overlay->text_fill_image =
+      g_malloc (4 * overlay->width * overlay->text_height);
+  cairo_set_target_image (overlay->cr, (gchar *) overlay->text_fill_image,
+      CAIRO_FORMAT_ARGB32, overlay->width, overlay->text_height,
+      overlay->width * 4);
 
   cairo_save (overlay->cr);
   cairo_rectangle (overlay->cr, 0, 0, overlay->width, overlay->text_height);
-  cairo_set_alpha (overlay->cr, 0.0);
+  cairo_set_source_rgba (overlay->cr, 0, 0, 0, 1.0);
+
   cairo_set_operator (overlay->cr, CAIRO_OPERATOR_SRC);
   cairo_fill (overlay->cr);
   cairo_restore (overlay->cr);
 
   cairo_save (overlay->cr);
-  cairo_text_extents (overlay->cr, string, &extents);
-  cairo_set_rgb_color (overlay->cr, 1, 1, 1);
-  cairo_move_to (overlay->cr, 0, overlay->text_height - 2);
-  cairo_show_text (overlay->cr, string);
-
-  cairo_text_path (overlay->cr, string);
-  cairo_set_rgb_color (overlay->cr, 0, 0, 0);
-  cairo_set_line_width (overlay->cr, 1.0);
-  cairo_stroke (overlay->cr);
-
-  g_free (string);
+  cairo_text_extents (overlay->cr, (guchar *) string, &extents);
+  cairo_set_source_rgba (overlay->cr, 1, 1, 1, 1.0);
 
+  switch (overlay->halign) {
+    case GST_TEXT_OVERLAY_HALIGN_LEFT:
+      x = overlay->x0;
+      break;
+    case GST_TEXT_OVERLAY_HALIGN_CENTER:
+      x = overlay->x0 - extents.width / 2;
+      break;
+    case GST_TEXT_OVERLAY_HALIGN_RIGHT:
+      x = overlay->x0 - extents.width;
+      break;
+    default:
+      x = 0;
+  }
+  y = overlay->text_height - 2;
+  cairo_move_to (overlay->cr, x, y);
+  cairo_show_text (overlay->cr, (guchar *) string);
   cairo_restore (overlay->cr);
 
-  overlay->text_width = MIN (extents.width, overlay->width);
+  if (overlay->text_outline_image)
+    g_free (overlay->text_outline_image);
+  overlay->text_outline_image =
+      g_malloc (4 * overlay->width * overlay->text_height);
+  cairo_set_target_image (overlay->cr, (gchar *) overlay->text_outline_image,
+      CAIRO_FORMAT_ARGB32, overlay->width, overlay->text_height,
+      overlay->width * 4);
+
+  cairo_save (overlay->cr);
+  cairo_rectangle (overlay->cr, 0, 0, overlay->width, overlay->text_height);
+  cairo_set_source_rgba (overlay->cr, 0, 0, 0, 1.0);
+  cairo_set_operator (overlay->cr, CAIRO_OPERATOR_SRC);
+  cairo_fill (overlay->cr);
+  cairo_restore (overlay->cr);
+
+  cairo_save (overlay->cr);
+  cairo_move_to (overlay->cr, x, y);
+  cairo_set_source_rgba (overlay->cr, 1, 1, 1, 1.0);
+  cairo_set_line_width (overlay->cr, 1.0);
+  cairo_text_path (overlay->cr, (guchar *) string);
+  cairo_stroke (overlay->cr);
+  cairo_restore (overlay->cr);
+
+  g_free (string);
 }
 
 /* static GstPadLinkReturn */
@@ -272,36 +315,47 @@ gst_textoverlay_video_sinkconnect (GstPad * pad, const GstCaps * caps)
 
 
 static void
-gst_text_overlay_blit_yuv420 (GstTextOverlay * overlay, guchar * pixbuf,
-    int x0, int y0)
+gst_text_overlay_blit_1 (GstTextOverlay * overlay, guchar * dest,
+    guchar * text_image, int val)
 {
   int i;
   int j;
   int x, a, y;
+  int y0 = 0;
 
-  if (x0 < 0)
-    x0 = 0;
-  if (y0 < 0)
-    y0 = 0;
-
-  GST_LOG ("%d %d %d %d %d %d %p", overlay->text_width, overlay->text_height,
-      overlay->width, overlay->height, x0, y0, pixbuf);
+  y = val;
 
   for (i = 0; i < overlay->text_height; i++) {
-    for (j = 0; j < overlay->text_width; j++) {
-#if 0
-      x = pixbuf[(i + y0) * overlay->width + (j + x0)];
-      a = overlay->pixbuf[4 * (i * overlay->width + j) + 0];
-      y = overlay->pixbuf[4 * (i * overlay->width + j) + 1];
-      pixbuf[(i + y0) * overlay->width + (j + x0)] =
-          (y * a + x * (255 - a)) / 255;
-#endif
-      x = a = 0;
-      y = overlay->pixbuf[4 * (i * overlay->width + j) + 1];
-      pixbuf[(i + y0) * overlay->width + (j + x0)] = y;
+    for (j = 0; j < overlay->width; j++) {
+      x = dest[(i + y0) * overlay->width + j];
+      a = text_image[4 * (i * overlay->width + j) + 1];
+      dest[(i + y0) * overlay->width + j] = (y * a + x * (255 - a)) / 255;
     }
   }
+}
 
+static void
+gst_text_overlay_blit_sub2x2 (GstTextOverlay * overlay, guchar * dest,
+    guchar * text_image, int val)
+{
+  int i;
+  int j;
+  int x, a, y;
+  int y0 = 0;
+
+  y = val;
+
+  for (i = 0; i < overlay->text_height; i += 2) {
+    for (j = 0; j < overlay->width; j += 2) {
+      x = dest[(i / 2 + y0) * (overlay->width / 2) + j / 2];
+      a = (text_image[4 * (i * overlay->width + j) + 1] +
+          text_image[4 * (i * overlay->width + j + 1) + 1] +
+          text_image[4 * ((i + 1) * overlay->width + j) + 1] +
+          text_image[4 * ((i + 1) * overlay->width + j + 1) + 1] + 2) / 4;
+      dest[(i / 2 + y0) * (overlay->width / 2) + j / 2] =
+          (y * a + x * (255 - a)) / 255;
+    }
+  }
 }
 
 
@@ -311,7 +365,7 @@ gst_textoverlay_video_chain (GstPad * pad, GstData * _data)
   GstBuffer *buf = GST_BUFFER (_data);
   GstTextOverlay *overlay;
   guchar *pixbuf;
-  gint x0, y0;
+  gint y;
 
   g_return_if_fail (pad != NULL);
   g_return_if_fail (GST_IS_PAD (pad));
@@ -325,32 +379,38 @@ gst_textoverlay_video_chain (GstPad * pad, GstData * _data)
 
   pixbuf = GST_BUFFER_DATA (buf);
 
-  x0 = overlay->x0;
-  y0 = overlay->y0;
+  y = overlay->y0;
   switch (overlay->valign) {
     case GST_TEXT_OVERLAY_VALIGN_BOTTOM:
-      y0 -= overlay->text_height;
+      y -= overlay->text_height;
       break;
     case GST_TEXT_OVERLAY_VALIGN_BASELINE:
 #define BASELINE 2
-      y0 -= (overlay->text_height - BASELINE);
+      y -= (overlay->text_height - BASELINE);
       break;
     case GST_TEXT_OVERLAY_VALIGN_TOP:
       break;
   }
 
-  switch (overlay->halign) {
-    case GST_TEXT_OVERLAY_HALIGN_LEFT:
-      break;
-    case GST_TEXT_OVERLAY_HALIGN_RIGHT:
-      x0 -= overlay->text_width;
-      break;
-    case GST_TEXT_OVERLAY_HALIGN_CENTER:
-      x0 -= overlay->text_width;
-      break;
-  }
+  gst_text_overlay_blit_1 (overlay,
+      pixbuf + y * overlay->width, overlay->text_outline_image, 0);
+  gst_text_overlay_blit_sub2x2 (overlay,
+      pixbuf + (overlay->height * overlay->width) +
+      (y / 2) * overlay->width / 2, overlay->text_outline_image, 128);
+  gst_text_overlay_blit_sub2x2 (overlay, pixbuf +
+      (overlay->height * overlay->width) +
+      (overlay->height * overlay->width) / 4 + (y / 2) * overlay->width / 2,
+      overlay->text_outline_image, 128);
 
-  gst_text_overlay_blit_yuv420 (overlay, pixbuf, x0, y0);
+  gst_text_overlay_blit_1 (overlay, pixbuf + y * overlay->width,
+      overlay->text_fill_image, 255);
+  gst_text_overlay_blit_sub2x2 (overlay,
+      pixbuf + (overlay->height * overlay->width) +
+      (y / 2) * overlay->width / 2, overlay->text_fill_image, 128);
+  gst_text_overlay_blit_sub2x2 (overlay,
+      pixbuf + (overlay->height * overlay->width) +
+      (overlay->height * overlay->width) / 4 + (y / 2) * overlay->width / 2,
+      overlay->text_fill_image, 128);
 
   gst_pad_push (overlay->srcpad, GST_DATA (buf));
 }
@@ -372,8 +432,22 @@ gst_textoverlay_loop (GstElement * element)
   g_return_if_fail (GST_IS_TEXTOVERLAY (element));
   overlay = GST_TEXTOVERLAY (element);
 
-  video_frame = GST_BUFFER (gst_pad_pull (overlay->video_sinkpad));
+  do {
+    GST_DEBUG ("Attempting to pull next video frame");
+    video_frame = GST_BUFFER (gst_pad_pull (overlay->video_sinkpad));
+    if (GST_IS_EVENT (video_frame)) {
+      GstEvent *event = GST_EVENT (video_frame);
+      GstEventType type = GST_EVENT_TYPE (event);
+
+      gst_pad_event_default (overlay->video_sinkpad, event);
+      GST_DEBUG ("Received event of type %d", type);
+      if (type == GST_EVENT_EOS || type == GST_EVENT_INTERRUPT)
+        return;
+      video_frame = NULL;
+    }
+  } while (!video_frame);
   now = GST_BUFFER_TIMESTAMP (video_frame);
+  GST_DEBUG ("Got video frame, time=%" GST_TIME_FORMAT, GST_TIME_ARGS (now));
 
   /*
    * This state machine has a bug that can't be resolved easily.
@@ -386,20 +460,41 @@ gst_textoverlay_loop (GstElement * element)
    * buffer timestamps and durations correctly.  (I think)
    */
 
-  while (overlay->next_buffer == NULL) {
+  while ((!overlay->current_buffer ||
+          PAST_END (overlay->current_buffer, now)) &&
+      overlay->next_buffer == NULL) {
     GST_DEBUG ("attempting to pull a buffer");
 
     /* read all text buffers until we get one "in the future" */
     if (!GST_PAD_IS_USABLE (overlay->text_sinkpad)) {
       break;
     }
-    overlay->next_buffer = GST_BUFFER (gst_pad_pull (overlay->text_sinkpad));
-    if (!overlay->next_buffer)
-      break;
+    do {
+      overlay->next_buffer = GST_BUFFER (gst_pad_pull (overlay->text_sinkpad));
+      if (GST_IS_EVENT (overlay->next_buffer)) {
+        GstEvent *event = GST_EVENT (overlay->next_buffer);
+        GstEventType type = GST_EVENT_TYPE (event);
+
+        gst_pad_event_default (overlay->text_sinkpad, event);
+        if (type == GST_EVENT_EOS || type == GST_EVENT_INTERRUPT)
+          return;
+        overlay->next_buffer = NULL;
+      }
+    } while (!overlay->next_buffer);
 
     if (PAST_END (overlay->next_buffer, now)) {
+      GST_DEBUG ("Received buffer is past end (%" GST_TIME_FORMAT " + %"
+          GST_TIME_FORMAT " < %" GST_TIME_FORMAT ")",
+          GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (overlay->next_buffer)),
+          GST_TIME_ARGS (GST_BUFFER_DURATION (overlay->next_buffer)),
+          GST_TIME_ARGS (now));
       gst_buffer_unref (overlay->next_buffer);
       overlay->next_buffer = NULL;
+    } else {
+      GST_DEBUG ("Received new text buffer of time %" GST_TIME_FORMAT
+          "and duration %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (overlay->next_buffer)),
+          GST_TIME_ARGS (GST_BUFFER_DURATION (overlay->next_buffer)));
     }
   }
 
@@ -418,7 +513,7 @@ gst_textoverlay_loop (GstElement * element)
         GST_BUFFER_SIZE (overlay->current_buffer),
         GST_BUFFER_DATA (overlay->current_buffer));
     gst_textoverlay_render_text (overlay,
-        GST_BUFFER_DATA (overlay->current_buffer),
+        (gchar *) GST_BUFFER_DATA (overlay->current_buffer),
         GST_BUFFER_SIZE (overlay->current_buffer));
     overlay->need_render = FALSE;
   }
@@ -443,6 +538,22 @@ gst_textoverlay_loop (GstElement * element)
   gst_textoverlay_video_chain (overlay->srcpad, GST_DATA (video_frame));
 }
 
+static void
+gst_textoverlay_font_init (GstTextOverlay * overlay)
+{
+  cairo_font_extents_t font_extents;
+
+  cairo_select_font (overlay->cr, overlay->font, overlay->slant,
+      overlay->weight);
+  cairo_scale_font (overlay->cr, overlay->scale);
+
+  cairo_current_font_extents (overlay->cr, &font_extents);
+  overlay->text_height = font_extents.height;
+  if (overlay->text_height & 1)
+    overlay->text_height++;
+
+  overlay->need_render = TRUE;
+}
 
 static GstElementStateReturn
 gst_textoverlay_change_state (GstElement * element)
@@ -480,8 +591,6 @@ gst_textoverlay_finalize (GObject * object)
 static void
 gst_textoverlay_init (GstTextOverlay * overlay)
 {
-  cairo_font_extents_t font_extents;
-
   /* video sink */
   overlay->video_sinkpad =
       gst_pad_new_from_template (gst_static_pad_template_get
@@ -506,21 +615,20 @@ gst_textoverlay_init (GstTextOverlay * overlay)
 
   overlay->halign = GST_TEXT_OVERLAY_HALIGN_CENTER;
   overlay->valign = GST_TEXT_OVERLAY_VALIGN_BASELINE;
-  overlay->x0 = overlay->y0 = 0;
+  overlay->x0 = overlay->y0 = 25;
 
   overlay->default_text = g_strdup ("");
   overlay->need_render = TRUE;
 
-  cairo_select_font (overlay->cr, "sans", 0, 0);
-  cairo_scale_font (overlay->cr, 50);
-
-  cairo_current_font_extents (overlay->cr, &font_extents);
-  overlay->text_height = font_extents.height;
+  overlay->font = g_strdup ("sans");
+  overlay->slant = CAIRO_FONT_SLANT_NORMAL;
+  overlay->weight = CAIRO_FONT_WEIGHT_NORMAL;
+  overlay->scale = 20;
+  gst_textoverlay_font_init (overlay);
 
   gst_element_set_loop_function (GST_ELEMENT (overlay), gst_textoverlay_loop);
 }
 
-
 static void
 gst_textoverlay_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -551,6 +659,7 @@ gst_textoverlay_set_property (GObject * object, guint prop_id,
       else
         g_warning ("Invalid 'valign' property value: %s",
             g_value_get_string (value));
+      overlay->need_render = TRUE;
       break;
 
     case ARG_HALIGN:
@@ -563,6 +672,7 @@ gst_textoverlay_set_property (GObject * object, guint prop_id,
       else
         g_warning ("Invalid 'halign' property value: %s",
             g_value_get_string (value));
+      overlay->need_render = TRUE;
       break;
 
     case ARG_X0:
@@ -574,22 +684,11 @@ gst_textoverlay_set_property (GObject * object, guint prop_id,
       break;
 
     case ARG_FONT_DESC:
-    {
-#if 0
-      PangoFontDescription *desc;
-
-      desc = pango_font_description_from_string (g_value_get_string (value));
-      if (desc) {
-        g_message ("font description set: %s", g_value_get_string (value));
-        pango_layout_set_font_description (overlay->layout, desc);
-        pango_font_description_free (desc);
-        render_text (overlay);
-      } else
-        g_warning ("font description parse failed: %s",
-            g_value_get_string (value));
-#endif
+      if (overlay->font)
+        g_free (overlay->font);
+      overlay->font = g_strdup (g_value_get_string (value));
+      gst_textoverlay_font_init (overlay);
       break;
-    }
 
     default:
       break;
