@@ -43,6 +43,9 @@
 #define CD_SECS              60 /* seconds per minute */
 #define CD_FRAMES            75 /* frames per second */
 
+GST_DEBUG_CATEGORY_STATIC (cdparanoia_debug);
+#define GST_CAT_DEFAULT cdparanoia_debug
+
 #include "gstcdparanoia.h"
 
 
@@ -277,6 +280,9 @@ cdparanoia_class_init (CDParanoiaClass * klass)
   gst_tag_register ("discid", GST_TAG_FLAG_META, G_TYPE_STRING,
       _("discid"), _("CDDA discid for metadata retrieval"),
       gst_tag_merge_use_first);
+  gst_tag_register ("musicbrainz-discid", GST_TAG_FLAG_META, G_TYPE_STRING,
+      _("musicbrainz-discid"), _("Musicbrainz discid for metadata retrieval"),
+      gst_tag_merge_use_first);
 
   gobject_class->set_property = cdparanoia_set_property;
   gobject_class->get_property = cdparanoia_get_property;
@@ -320,8 +326,10 @@ cdparanoia_init (CDParanoia * cdparanoia)
   cdparanoia->total_seconds = 0;
   cdparanoia->discont_sent = FALSE;
   cdparanoia->uri = NULL;
-  cdparanoia->uri_track = -1;
+  cdparanoia->cur_track = -1;
   cdparanoia->seek_request = -1;
+  cdparanoia->track_mode = FALSE;
+  cdparanoia->pending_metadata = NULL;
 }
 
 static void
@@ -448,7 +456,7 @@ cdparanoia_get_property (GObject * object, guint prop_id, GValue * value,
       break;
     case ARG_DISCID:
       /**
-       * Due to possible autocorrections of start sectors of audio tracks on 
+       * Due to possible autocorrections of start sectors of audio tracks on
        * multisession cds, we can maybe not compute the correct discid.
        * So issue a warning.
        * See cdparanoia/interface/common-interface.c:FixupTOC
@@ -469,6 +477,55 @@ cdparanoia_callback (long inpos, int function)
 {
 }
 
+static guint
+get_first_sector (CDParanoia * src, guint track)
+{
+  if (!src->track_mode)
+    return 0;
+  else if (track == src->d->tracks)
+    return src->last_sector + 1;
+  else
+    return cdda_track_firstsector (src->d, track + 1);
+}
+
+static guint
+get_relative (CDParanoia * src, guint track, guint sector)
+{
+  return sector - get_first_sector (src, track);
+}
+
+static void
+set_track (CDParanoia * src, guint track)
+{
+  src->cur_track = track;
+  g_free (src->uri);
+  src->uri = g_strdup_printf ("cdda://%d", track++);
+}
+
+static gboolean
+is_track_switch (CDParanoia * src, guint sector)
+{
+  gint n;
+
+  if (!src->track_mode)
+    return FALSE;
+
+  if (sector < get_first_sector (src, src->cur_track) ||
+      sector >= get_first_sector (src, src->cur_track + 1)) {
+    /* get track */
+    for (n = 0; n < src->d->tracks; n++) {
+      if (sector >= get_first_sector (src, n) &&
+          sector < get_first_sector (src, n + 1))
+        break;
+    }
+    set_track (src, n);
+
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
 static GstData *
 cdparanoia_get (GstPad * pad)
 {
@@ -480,13 +537,20 @@ cdparanoia_get (GstPad * pad)
   g_return_val_if_fail (GST_FLAG_IS_SET (src, CDPARANOIA_OPEN), NULL);
 
   if (src->seek_request != -1) {
-    gst_pad_send_event (src->srcpad,
+    cdparanoia_event (src->srcpad,
         gst_event_new_segment_seek (track_format |
             GST_SEEK_METHOD_SET |
             GST_SEEK_FLAG_FLUSH, src->seek_request - 1, src->seek_request));
     src->seek_request = -1;
   }
 
+  if (src->pending_metadata) {
+    GstData *d = GST_DATA (src->pending_metadata);
+
+    src->pending_metadata = NULL;
+    return d;
+  }
+
   /* stop things apropriatly */
   if (src->cur_sector > src->segment_end_sector) {
     GST_DEBUG ("setting EOS");
@@ -501,12 +565,22 @@ cdparanoia_get (GstPad * pad)
     /* convert the sequence sector number to a timestamp */
     format = GST_FORMAT_TIME;
     timestamp = 0LL;
-    gst_pad_convert (src->srcpad, sector_format, src->cur_sector, &format,
+    gst_pad_convert (src->srcpad, sector_format,
+        get_relative (src, src->cur_track, src->cur_sector), &format,
         &timestamp);
 
-    if (!src->discont_sent && src->prev_sec != src->cur_sector) {
+    if (!src->discont_sent && (is_track_switch (src, src->cur_sector) ||
+            (src->prev_sec != src->cur_sector))) {
+      gst_pad_convert (src->srcpad, sector_format,
+          get_relative (src, src->cur_track, src->cur_sector), &format,
+          &timestamp);
       GstEvent *discont_ev;
 
+      if (src->flush_pending) {
+        src->flush_pending = FALSE;
+        return GST_DATA (gst_event_new (GST_EVENT_FLUSH));
+      }
+
       discont_ev =
           gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, timestamp,
           GST_FORMAT_UNDEFINED);
@@ -605,10 +679,42 @@ cddb_discid (gchar * discid, toc_msf * toc, gint tracks)
   sprintf (discid, "%08x", ((n % 0xff) << 24 | t << 8 | tracks));
 }
 
+#include "base64.h"
+#include "sha1.h"
+
+static void
+get_musicbrainz_discid (toc_msf * toc, TOC * xtoc, gint tracks, char *id)
+{
+  guchar digest[20], *ptr;
+  gchar tmp[9];
+  gulong i;
+  SHA_INFO sha;
+
+  /* generate SHA digest */
+  sha_init (&sha);
+  sprintf (tmp, "%02X", xtoc[0].bTrack);
+  sha_update (&sha, (SHA_BYTE *) tmp, 2);
+  sprintf (tmp, "%02X", xtoc[tracks - 1].bTrack);
+  sha_update (&sha, (SHA_BYTE *) tmp, 2);
+  sprintf (tmp, "%08X", toc[tracks].f);
+  sha_update (&sha, (SHA_BYTE *) tmp, 8);
+  for (i = 0; i < 99; i++) {
+    sprintf (tmp, "%08X", (i < tracks) ? toc[i].f : 0);
+    sha_update (&sha, (SHA_BYTE *) tmp, 8);
+  }
+  sha_final (digest, &sha);
+
+  /* re-encode to base64 */
+  ptr = rfc822_binary (digest, 20, &i);
+  memcpy (id, ptr, i);
+  id[i] = '\0';
+  free (ptr);
+}
+
 /* get all the cddb info at once */
 static void
-get_cddb_info (TOC * toc, gint tracks, gchar * discid, gint64 * offsets,
-    gint64 * total_seconds)
+get_cddb_info (TOC * toc, gint tracks, gchar * discid,
+    gchar * musicbrainz_discid, gint64 * offsets, gint64 * total_seconds)
 {
   toc_msf msf_toc[MAXTRK];
   gint i;
@@ -623,6 +729,7 @@ get_cddb_info (TOC * toc, gint tracks, gchar * discid, gint64 * offsets,
 
   *total_seconds = msf_toc[tracks].f / 75;
 
+  get_musicbrainz_discid (msf_toc, toc, tracks, musicbrainz_discid);
 }
 
 static void
@@ -633,7 +740,7 @@ add_index_associations (CDParanoia * src)
   for (i = 0; i < src->d->tracks; i++) {
     gint64 sector;
 
-    sector = cdda_track_firstsector (src->d, i + 1);
+    sector = get_first_sector (src, i);
     gst_index_add_association (src->index, src->index_id,
         GST_ASSOCIATION_FLAG_KEY_UNIT,
         track_format, i,
@@ -661,6 +768,7 @@ cdparanoia_open (CDParanoia * src)
   GstTagList *taglist;
   gint i;
   gint paranoia_mode;
+  char musicbrainz_discid[33];
 
   g_return_val_if_fail (!GST_FLAG_IS_SET (src, CDPARANOIA_OPEN), FALSE);
 
@@ -706,20 +814,21 @@ cdparanoia_open (CDParanoia * src)
    * when leaving the function it clobbers the allocated mem and all is lost bugger
    */
 
-  get_cddb_info (&src->d->disc_toc[0], src->d->tracks, src->discid,
-      src->offsets, &src->total_seconds);
+  get_cddb_info (&src->d->disc_toc[0], src->d->tracks - src->d->cd_extra,
+      src->discid, musicbrainz_discid, src->offsets, &src->total_seconds);
 
   g_object_freeze_notify (G_OBJECT (src));
   g_object_notify (G_OBJECT (src), "discid");
   g_object_thaw_notify (G_OBJECT (src));
 
   taglist = gst_tag_list_new ();
-  gst_tag_list_add (taglist, GST_TAG_MERGE_APPEND, "discid", src->discid, NULL);
+  gst_tag_list_add (taglist, GST_TAG_MERGE_APPEND, "discid", src->discid,
+      "musicbrainz-discid", musicbrainz_discid, NULL);
   gst_element_found_tags (GST_ELEMENT (src), taglist);
-  /* no need to forward, because discid is useless to other elements */
+  src->pending_metadata = gst_event_new_tag (taglist);
 
   if (src->toc_bias) {
-    src->toc_offset -= cdda_track_firstsector (src->d, 1);
+    src->toc_offset -= get_first_sector (src, 0);
   }
   for (i = 0; i < src->d->tracks + 1; i++) {
     src->d->disc_toc[i].dwStartSector += src->toc_offset;
@@ -730,8 +839,13 @@ cdparanoia_open (CDParanoia * src)
   }
 
   /* save thse ones - skip lead-in */
-  src->first_sector = 0; //cdda_track_firstsector (src->d, 1);
-  src->last_sector = cdda_disc_lastsector (src->d);
+  if (src->track_mode) {
+    src->first_sector = get_first_sector (src, src->cur_track);
+    src->last_sector = get_first_sector (src, src->cur_track + 1) - 1;
+  } else {
+    src->first_sector = cdda_track_firstsector (src->d, 1);
+    src->last_sector = cdda_disc_lastsector (src->d);
+  }
 
   /* this is the default segment we will play */
   src->segment_start_sector = src->first_sector;
@@ -805,9 +919,6 @@ cdparanoia_change_state (GstElement * element)
       }
       break;
     case GST_STATE_READY_TO_PAUSED:
-      if (cdparanoia->uri_track > 0) {
-        cdparanoia->seek_request = cdparanoia->uri_track;
-      }
       break;
     case GST_STATE_PAUSED_TO_PLAYING:
       break;
@@ -815,6 +926,10 @@ cdparanoia_change_state (GstElement * element)
       break;
     case GST_STATE_PAUSED_TO_READY:
       /* FIXME: Need code here to reset element to start of cd */
+      if (cdparanoia->pending_metadata) {
+        gst_event_unref (cdparanoia->pending_metadata);
+        cdparanoia->pending_metadata = NULL;
+      }
       break;
     case GST_STATE_READY_TO_NULL:
       cdparanoia_close (CDPARANOIA (element));
@@ -870,51 +985,99 @@ cdparanoia_event (GstPad * pad, GstEvent * event)
       offset = GST_EVENT_SEEK_OFFSET (event);
       endoffset = GST_EVENT_SEEK_ENDOFFSET (event);
 
-      /* we can only seek on sectors, so we convert the requested
-       * offsets to sectors first */
-      if (offset != -1) {
-        res &= gst_pad_convert (src->srcpad, format, offset,
-            &sector_format, &seg_start_sector);
-      }
-      if (endoffset != -1) {
-        res &= gst_pad_convert (src->srcpad, format, endoffset,
-            &sector_format, &seg_end_sector);
-      }
+      if (format == track_format) {
+        gint track = 0;
 
-      if (!res) {
-        GST_DEBUG ("could not convert offsets to sectors");
-        goto error;
-      }
-
-      switch (GST_EVENT_SEEK_METHOD (event)) {
-        case GST_SEEK_METHOD_SET:
-          /* values are set for regular seek set */
-          break;
-        case GST_SEEK_METHOD_CUR:
-          if (seg_start_sector != -1) {
-            seg_start_sector += src->cur_sector;
-          }
-          if (seg_end_sector != -1) {
-            seg_end_sector += src->cur_sector;
-          }
-          break;
-        case GST_SEEK_METHOD_END:
-          if (seg_start_sector != -1) {
-            seg_start_sector = src->last_sector - seg_start_sector;
-          }
-          if (seg_end_sector != -1) {
-            seg_end_sector = src->last_sector - seg_end_sector;
-          }
-          break;
-        default:
+        switch (GST_EVENT_SEEK_METHOD (event)) {
+          case GST_SEEK_METHOD_SET:
+            track = offset;
+            break;
+          case GST_SEEK_METHOD_END:
+            track = src->d->tracks - offset;
+            break;
+          case GST_SEEK_METHOD_CUR:
+            track = src->cur_track + offset;
+            break;
+        }
+        if (track < 0 || track >= src->d->tracks) {
+          GST_WARNING ("track %d is out of range (%d avail)",
+              track, src->d->tracks);
           goto error;
+        }
+
+        seg_start_sector = get_first_sector (src, track);
+        GST_LOG ("Seeking to track %d (sector %" G_GINT64_FORMAT ")",
+            track, seg_start_sector);
+      } else {
+        /* from here on, all seeks are relative to the current track */
+
+        /* we can only seek on sectors, so we convert the requested
+         * offsets to sectors first */
+        if (offset != -1) {
+          res &= gst_pad_convert (src->srcpad, format, offset,
+              &sector_format, &seg_start_sector);
+        }
+        if (endoffset != -1) {
+          res &= gst_pad_convert (src->srcpad, format, endoffset,
+              &sector_format, &seg_end_sector);
+        }
+
+        if (!res) {
+          GST_DEBUG ("could not convert offsets to sectors");
+          goto error;
+        }
+
+        switch (GST_EVENT_SEEK_METHOD (event)) {
+          case GST_SEEK_METHOD_SET:
+            /* values are set for regular seek set */
+            if (seg_start_sector != -1) {
+              seg_start_sector += get_first_sector (src, src->cur_track);
+            }
+            if (seg_end_sector != -1) {
+              seg_end_sector += get_first_sector (src, src->cur_track);
+            }
+            break;
+          case GST_SEEK_METHOD_CUR:
+            if (seg_start_sector != -1) {
+              seg_start_sector += src->cur_sector;
+            }
+            if (seg_end_sector != -1) {
+              seg_end_sector += src->cur_sector;
+            }
+            break;
+          case GST_SEEK_METHOD_END:
+            if (seg_start_sector != -1) {
+              seg_start_sector += get_first_sector (src, src->cur_track + 1);
+            }
+            if (seg_end_sector != -1) {
+              seg_end_sector += get_first_sector (src, src->cur_track + 1);
+            }
+            break;
+          default:
+            goto error;
+        }
+
+        if (seg_start_sector != -1) {
+          /* make sure it's in range */
+          if (seg_start_sector < get_first_sector (src, src->cur_track) ||
+              seg_start_sector >= get_first_sector (src, src->cur_track + 1)) {
+            GST_WARNING ("Seek out of range (sector %" G_GINT64_FORMAT
+                " is not in track %d)", seg_start_sector, src->cur_track);
+            goto error;
+          }
+        }
+        if (seg_end_sector != -1) {
+          if (seg_end_sector < get_first_sector (src, src->cur_track) ||
+              seg_end_sector >= get_first_sector (src, src->cur_track + 1)) {
+            GST_WARNING ("Seek out of range (sector %" G_GINT64_FORMAT
+                " is not in track %d)", seg_end_sector, src->cur_track);
+            seg_end_sector = -1;
+          }
+        }
       }
+
       /* do we need to update the start sector? */
       if (seg_start_sector != -1) {
-        seg_start_sector += src->first_sector;
-        seg_start_sector = CLAMP (seg_start_sector,
-            src->first_sector, src->last_sector);
-
         if (paranoia_seek (src->p, seg_start_sector, SEEK_SET) > -1) {
           GST_DEBUG ("seeked to %" G_GINT64_FORMAT, seg_start_sector);
 
@@ -925,11 +1088,9 @@ cdparanoia_event (GstPad * pad, GstEvent * event)
         }
       }
       if (seg_end_sector != -1) {
-        seg_end_sector += src->first_sector;
-        seg_end_sector = CLAMP (seg_end_sector,
-            src->first_sector, src->last_sector);
         src->segment_end_sector = seg_end_sector;
       }
+      src->flush_pending = GST_EVENT_SEEK_FLAGS (event) & GST_SEEK_FLAG_FLUSH;
       GST_DEBUG ("configured for %d -> %d sectors\n",
           src->segment_start_sector, src->segment_end_sector);
       break;
@@ -1034,7 +1195,7 @@ cdparanoia_convert (GstPad * pad,
         if (src_value < 0 || src_value > src->d->tracks)
           return FALSE;
 
-        sector = cdda_track_firstsector (src->d, src_value + 1);
+        sector = get_first_sector (src, src_value);
       } else if (src_format == sector_format) {
         sector = src_value;
       } else {
@@ -1103,14 +1264,24 @@ cdparanoia_query (GstPad * pad, GstQueryType type,
     case GST_QUERY_TOTAL:
       /* we take the last sector + 1 so that we also have the full
        * size of that last sector */
-      res = gst_pad_convert (src->srcpad,
-          sector_format, src->last_sector + 1 - src->first_sector,
-          format, value);
+      if (*format == track_format) {
+        *value = src->d->tracks;
+        res = TRUE;
+      } else {
+        res = gst_pad_convert (src->srcpad, sector_format,
+            get_first_sector (src, src->cur_track + 1) -
+            get_first_sector (src, src->cur_track), format, value);
+      }
       break;
     case GST_QUERY_POSITION:
       /* bring our current sector to the requested format */
-      res = gst_pad_convert (src->srcpad,
-          sector_format, src->cur_sector - src->first_sector, format, value);
+      if (*format == track_format) {
+        *value = src->cur_track;
+        res = TRUE;
+      } else {
+        res = gst_pad_convert (src->srcpad, sector_format,
+            get_relative (src, src->cur_track, src->cur_sector), format, value);
+      }
       break;
     case GST_QUERY_START:
       res = gst_pad_convert (src->srcpad,
@@ -1157,6 +1328,9 @@ cdparanoia_get_index (GstElement * element)
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
+  GST_DEBUG_CATEGORY_INIT (cdparanoia_debug, "cdparanoia", 0,
+      "CDDA reading through the cdparanoia library");
+
   if (!gst_element_register (plugin, "cdparanoia", GST_RANK_NONE,
           GST_TYPE_CDPARANOIA))
     return FALSE;
@@ -1203,9 +1377,11 @@ cdparanoia_uri_set_uri (GstURIHandler * handler, const gchar * uri)
   }
   g_free (protocol);
   location = gst_uri_get_location (uri);
-  cdparanoia->uri_track = strtol (location, NULL, 10);
-  if (cdparanoia->uri_track > 0) {
-    cdparanoia->seek_request = cdparanoia->uri_track;
+  if (sscanf (location, "%d", &cdparanoia->cur_track) == 1) {
+    cdparanoia->track_mode = TRUE;
+    cdparanoia->cur_track--;
+  } else {
+    cdparanoia->track_mode = FALSE;
   }
   g_free (location);
 
