@@ -87,7 +87,7 @@ struct _GstFlacTagClass
 
 /* elementfactory information */
 static GstElementDetails gst_flac_tag_details =
-GST_ELEMENT_DETAILS ("flac rettager",
+GST_ELEMENT_DETAILS ("flac retagger",
     "Tag",
     "Rewrite tags in a FLAC file",
     "Christope Fergeau <teuf@gnome.org>");
@@ -295,7 +295,7 @@ gst_flac_tag_chain (GstPad * pad, GstData * data)
       return;
     }
 
-    if (strncmp (GST_BUFFER_DATA (tag->buffer), FLAC_MAGIC,
+    if (strncmp ((gchar *) GST_BUFFER_DATA (tag->buffer), FLAC_MAGIC,
             FLAC_MAGIC_SIZE) == 0) {
       GstBuffer *sub;
 
