@@ -71,8 +71,8 @@ enum
 
 enum
 {
-  ARG_0
-      /* FILL ME */
+  ARG_0,
+  ARG_SKIP_FRAME
 };
 
 /*
@@ -228,6 +228,28 @@ gst_mpeg2dec_base_init (gpointer g_class)
   gst_element_class_set_details (element_class, &gst_mpeg2dec_details);
 }
 
+#define GST_MPEG2DEC_TYPE_SKIPFRAME (gst_mpeg2dec_skipframe_get_type())
+static GType
+gst_mpeg2dec_skipframe_get_type (void)
+{
+  static GType mpeg2dec_skipframe_type = 0;
+
+  if (!mpeg2dec_skipframe_type) {
+    static GEnumValue mpeg2dec_skipframe[] = {
+      {0, "0", "Skip nothing"},
+      {1, "1", "Skip B-frames"},
+      {2, "2", "Skip B-/P-frames"},
+      {3, "3", "Skip everything"},
+      {0, NULL, NULL},
+    };
+
+    mpeg2dec_skipframe_type =
+        g_enum_register_static ("GstMpeg2decSkipFrame", mpeg2dec_skipframe);
+  }
+
+  return mpeg2dec_skipframe_type;
+}
+
 static void
 gst_mpeg2dec_class_init (GstMpeg2decClass * klass)
 {
@@ -243,6 +265,11 @@ gst_mpeg2dec_class_init (GstMpeg2decClass * klass)
   gobject_class->get_property = gst_mpeg2dec_get_property;
   gobject_class->dispose = gst_mpeg2dec_dispose;
 
+  g_object_class_install_property (gobject_class, ARG_SKIP_FRAME,
+      g_param_spec_enum ("skip-frame", "Skip frames",
+          "Which types of frames to skip during decoding",
+          GST_MPEG2DEC_TYPE_SKIPFRAME, 0, G_PARAM_READWRITE));
+
   gstelement_class->change_state = gst_mpeg2dec_change_state;
   gstelement_class->set_index = gst_mpeg2dec_set_index;
   gstelement_class->get_index = gst_mpeg2dec_get_index;
@@ -294,6 +321,8 @@ gst_mpeg2dec_init (GstMpeg2dec * mpeg2dec)
   mpeg2dec->have_fbuf = FALSE;
   mpeg2dec->offset = 0;
 
+  mpeg2dec->skip_frame = 0;
+
   for (i = 0; i < GST_MPEG2DEC_NUM_BUFS; i++)
     mpeg2dec->buffers[i] = NULL;
 
@@ -561,7 +590,7 @@ static gboolean
 gst_mpeg2dec_negotiate_format (GstMpeg2dec * mpeg2dec)
 {
   GstCaps *caps;
-  guint32 fourcc, myFourcc;
+  guint32 myFourcc;
   gboolean ret;
   const mpeg2_info_t *info;
   const mpeg2_sequence_t *sequence;
@@ -594,16 +623,14 @@ gst_mpeg2dec_negotiate_format (GstMpeg2dec * mpeg2dec)
       "framerate", G_TYPE_DOUBLE, mpeg2dec->frame_rate, NULL);
 
   ret = gst_pad_set_explicit_caps (mpeg2dec->srcpad, caps);
+  gst_caps_free (caps);
+
   if (!ret)
     return FALSE;
 
-  /* it worked, try to find what it was again */
-  gst_structure_get_fourcc (gst_caps_get_structure (caps, 0),
-      "format", &fourcc);
-
-  if (fourcc == GST_STR_FOURCC ("Y42B")) {
+  if (myFourcc == GST_STR_FOURCC ("Y42B")) {
     mpeg2dec->format = MPEG2DEC_FORMAT_I422;
-  } else if (fourcc == GST_STR_FOURCC ("I420")) {
+  } else if (myFourcc == GST_STR_FOURCC ("I420")) {
     mpeg2dec->format = MPEG2DEC_FORMAT_I420;
   } else {
     mpeg2dec->format = MPEG2DEC_FORMAT_YV12;
@@ -664,11 +691,13 @@ handle_picture (GstMpeg2dec * mpeg2dec, const mpeg2_info_t * info)
 {
   gboolean key_frame = FALSE;
   GstBuffer *outbuf;
+  gint pic_type = 0;
 
   if (info->current_picture) {
     key_frame =
         (info->current_picture->flags & PIC_MASK_CODING_TYPE) ==
         PIC_FLAG_CODING_TYPE_I;
+    pic_type = info->current_picture->flags & PIC_MASK_CODING_TYPE;
   }
   outbuf = gst_mpeg2dec_alloc_buffer (mpeg2dec, mpeg2dec->offset);
   if (!outbuf)
@@ -682,7 +711,10 @@ handle_picture (GstMpeg2dec * mpeg2dec, const mpeg2_info_t * info)
   if (mpeg2dec->discont_state == MPEG2DEC_DISC_NEW_PICTURE && key_frame)
     mpeg2dec->discont_state = MPEG2DEC_DISC_NEW_KEYFRAME;
 
-  if (!GST_PAD_IS_USABLE (mpeg2dec->srcpad))
+  if (!GST_PAD_IS_USABLE (mpeg2dec->srcpad) ||
+      mpeg2dec->skip_frame == 3 ||
+      (mpeg2dec->skip_frame == 2 && pic_type != PIC_FLAG_CODING_TYPE_I) ||
+      (mpeg2dec->skip_frame == 1 && pic_type == PIC_FLAG_CODING_TYPE_B))
     mpeg2_skip (mpeg2dec->decoder, 1);
   else
     mpeg2_skip (mpeg2dec->decoder, 0);
@@ -809,6 +841,11 @@ handle_slice (GstMpeg2dec * mpeg2dec, const mpeg2_info_t * info)
       } else {
         GST_BUFFER_FLAG_UNSET (outbuf, GST_BUFFER_KEY_UNIT);
       }
+      GST_DEBUG_OBJECT (mpeg2dec, "pushing video buffer, timestamp %"
+          GST_TIME_FORMAT ", duration %" GST_TIME_FORMAT,
+          gst_element_get_name (mpeg2dec),
+          GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuf)),
+          GST_TIME_ARGS (GST_BUFFER_DURATION (outbuf)));
       gst_pad_push (mpeg2dec->srcpad, GST_DATA (outbuf));
     }
   } else if (info->display_fbuf && !info->display_fbuf->id) {
@@ -944,7 +981,7 @@ gst_mpeg2dec_chain (GstPad * pad, GstData * _data)
     switch (GST_EVENT_TYPE (event)) {
       case GST_EVENT_DISCONTINUOUS:
       {
-        GstClockTime time;
+        gint64 time;
 
         if (!gst_event_discont_get_value (event, GST_FORMAT_TIME, &time)
             || !GST_CLOCK_TIME_IS_VALID (time)) {
@@ -975,6 +1012,30 @@ gst_mpeg2dec_chain (GstPad * pad, GstData * _data)
         gst_pad_event_default (pad, event);
         return;
       }
+      case GST_EVENT_FILLER:{
+        /* Transform filler to always have timestamp + duration */
+        GstClockTime end_ts = GST_EVENT_TIMESTAMP (event);
+        GstClockTime dur = gst_event_filler_get_duration (event);
+
+        if (!GST_CLOCK_TIME_IS_VALID (end_ts))
+          end_ts = mpeg2dec->next_time;
+
+        if (GST_CLOCK_TIME_IS_VALID (dur))
+          end_ts += dur;
+
+        dur = GST_CLOCK_DIFF (end_ts, mpeg2dec->next_time);
+
+        gst_event_unref (event);
+
+        if (dur <= 0)
+          return;
+
+        event = gst_event_new_filler_stamped (mpeg2dec->next_time, dur);
+        mpeg2dec->next_time = end_ts;
+
+        gst_pad_event_default (pad, event);
+        return;
+      }
       case GST_EVENT_EOS:
         if (mpeg2dec->index && mpeg2dec->closed) {
           gst_index_commit (mpeg2dec->index, mpeg2dec->index_id);
@@ -1541,6 +1602,9 @@ gst_mpeg2dec_set_property (GObject * object, guint prop_id,
   src = GST_MPEG2DEC (object);
 
   switch (prop_id) {
+    case ARG_SKIP_FRAME:
+      src->skip_frame = g_value_get_enum (value);
+      break;
     default:
       break;
   }
@@ -1557,6 +1621,9 @@ gst_mpeg2dec_get_property (GObject * object, guint prop_id, GValue * value,
   mpeg2dec = GST_MPEG2DEC (object);
 
   switch (prop_id) {
+    case ARG_SKIP_FRAME:
+      g_value_set_enum (value, mpeg2dec->skip_frame);
+      break;
     default:
       break;
   }
