@@ -95,6 +95,7 @@ struct _GstTheoraEnc
   gint keyframe_threshold;
   gint keyframe_mindistance;
   gint noise_sensitivity;
+  gint sharpness;
 
   gint info_width, info_height;
   gint width, height;
@@ -126,6 +127,7 @@ struct _GstTheoraEncClass
 #define THEORA_DEF_KEYFRAME_THRESHOLD	80
 #define THEORA_DEF_KEYFRAME_MINDISTANCE	8
 #define THEORA_DEF_NOISE_SENSITIVITY	1
+#define THEORA_DEF_SHARPNESS		0
 
 enum
 {
@@ -141,6 +143,7 @@ enum
   ARG_KEYFRAME_THRESHOLD,
   ARG_KEYFRAME_MINDISTANCE,
   ARG_NOISE_SENSITIVITY,
+  ARG_SHARPNESS,
   /* FILL ME */
 };
 
@@ -243,6 +246,10 @@ gst_theora_enc_class_init (GstTheoraEncClass * klass)
       g_param_spec_int ("noise-sensitivity", "Noise sensitivity",
           "Noise sensitivity", 0, 32768, THEORA_DEF_NOISE_SENSITIVITY,
           (GParamFlags) G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, ARG_SHARPNESS,
+      g_param_spec_int ("sharpness", "Sharpness",
+          "Sharpness", 0, 2, THEORA_DEF_SHARPNESS,
+          (GParamFlags) G_PARAM_READWRITE));
 
   gstelement_class->change_state = theora_enc_change_state;
   GST_DEBUG_CATEGORY_INIT (theoraenc_debug, "theoraenc", 0, "Theora encoder");
@@ -276,6 +283,7 @@ gst_theora_enc_init (GstTheoraEnc * enc)
   enc->keyframe_threshold = THEORA_DEF_KEYFRAME_THRESHOLD;
   enc->keyframe_mindistance = THEORA_DEF_KEYFRAME_MINDISTANCE;
   enc->noise_sensitivity = THEORA_DEF_NOISE_SENSITIVITY;
+  enc->sharpness = THEORA_DEF_SHARPNESS;
 
   GST_FLAG_SET (enc, GST_ELEMENT_EVENT_AWARE);
 }
@@ -308,8 +316,8 @@ theora_enc_sink_link (GstPad * pad, const GstCaps * caps)
   /* center image if needed */
   if (enc->center) {
     /* make sure offset is even, for easier decoding */
-    enc->offset_x = ROUND_UP_2 (enc->info_width - enc->width) / 2;
-    enc->offset_y = ROUND_UP_2 (enc->info_height - enc->height) / 2;
+    enc->offset_x = ROUND_UP_2 ((enc->info_width - enc->width) / 2);
+    enc->offset_y = ROUND_UP_2 ((enc->info_height - enc->height) / 2);
   } else {
     enc->offset_x = 0;
     enc->offset_y = 0;
@@ -349,6 +357,7 @@ theora_enc_sink_link (GstPad * pad, const GstCaps * caps)
   enc->info.keyframe_auto_threshold = enc->keyframe_threshold;
   enc->info.keyframe_mindistance = enc->keyframe_mindistance;
   enc->info.noise_sensitivity = enc->noise_sensitivity;
+  enc->info.sharpness = enc->sharpness;
 
   theora_encode_init (&enc->state, &enc->info);
 
@@ -468,7 +477,7 @@ theora_enc_chain (GstPad * pad, GstData * data)
     switch (GST_EVENT_TYPE (data)) {
       case GST_EVENT_DISCONTINUOUS:
       {
-        guint64 val;
+        gint64 val;
 
         if (gst_event_discont_get_value (GST_EVENT (data), GST_FORMAT_TIME,
                 &val)) {
@@ -478,9 +487,9 @@ theora_enc_chain (GstPad * pad, GstData * data)
            * to spec? */
           if (enc->bytes_out == 0) {
             enc->initial_delay = val;
-            GST_DEBUG ("initial delay = %" G_GUINT64_FORMAT, val);
+            GST_DEBUG ("initial delay = %" G_GINT64_FORMAT, val);
           } else {
-            GST_DEBUG ("mid stream discont: val = %" G_GUINT64_FORMAT, val);
+            GST_DEBUG ("mid stream discont: val = %" G_GINT64_FORMAT, val);
           }
         }
         gst_pad_event_default (pad, GST_EVENT (data));
@@ -767,6 +776,9 @@ theora_enc_set_property (GObject * object, guint prop_id,
     case ARG_NOISE_SENSITIVITY:
       enc->noise_sensitivity = g_value_get_int (value);
       break;
+    case ARG_SHARPNESS:
+      enc->sharpness = g_value_get_int (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -813,6 +825,9 @@ theora_enc_get_property (GObject * object, guint prop_id,
     case ARG_NOISE_SENSITIVITY:
       g_value_set_int (value, enc->noise_sensitivity);
       break;
+    case ARG_SHARPNESS:
+      g_value_set_int (value, enc->sharpness);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
