@@ -66,42 +66,43 @@ enum
       "signed = (boolean) true")
 #if 0
   /* disabled because it segfaults */
-"audio/x-raw-float, "
-    "rate = (int) [ 1, MAX ], "
-    "channels = (int) [ 1, MAX ], "
+#define NOTHING "audio/x-raw-float, " \
+    "rate = (int) [ 1, MAX ], " \
+    "channels = (int) [ 1, MAX ], " \
     "endianness = (int) BYTE_ORDER, " "width = (int) 32")
 #endif
-     static GstStaticPadTemplate gst_audioscale_sink_template =
-         GST_STATIC_PAD_TEMPLATE ("sink",
-         GST_PAD_SINK, GST_PAD_ALWAYS, SUPPORTED_CAPS);
+static GstStaticPadTemplate gst_audioscale_sink_template =
+GST_STATIC_PAD_TEMPLATE ("sink",
+    GST_PAD_SINK, GST_PAD_ALWAYS, SUPPORTED_CAPS);
 
-     static GstStaticPadTemplate gst_audioscale_src_template =
-         GST_STATIC_PAD_TEMPLATE ("src",
-         GST_PAD_SRC, GST_PAD_ALWAYS, SUPPORTED_CAPS);
+static GstStaticPadTemplate gst_audioscale_src_template =
+GST_STATIC_PAD_TEMPLATE ("src",
+    GST_PAD_SRC, GST_PAD_ALWAYS, SUPPORTED_CAPS);
 
 #define GST_TYPE_AUDIOSCALE_METHOD (gst_audioscale_method_get_type())
-     static GType gst_audioscale_method_get_type (void)
-     {
-       static GType audioscale_method_type = 0;
-       static GEnumValue audioscale_methods[] =
+static GType
+gst_audioscale_method_get_type (void)
 {
-  {
-  GST_RESAMPLE_NEAREST, "0", "Nearest"}
-  ,
-  {
-  GST_RESAMPLE_BILINEAR, "1", "Bilinear"}
-  , {
-  GST_RESAMPLE_SINC, "2", "Sinc"}
-  , {
-  0, NULL, NULL}
-,};
+  static GType audioscale_method_type = 0;
+  static GEnumValue audioscale_methods[] = {
+    {
+        GST_RESAMPLE_NEAREST, "0", "Nearest"}
+    ,
+    {
+        GST_RESAMPLE_BILINEAR, "1", "Bilinear"}
+    , {
+        GST_RESAMPLE_SINC, "2", "Sinc"}
+    , {
+        0, NULL, NULL}
+    ,
+  };
 
-       if (!audioscale_method_type) {
-         audioscale_method_type = g_enum_register_static ("GstAudioscaleMethod",
-             audioscale_methods);
-       }
-       return audioscale_method_type;
-     }
+  if (!audioscale_method_type) {
+    audioscale_method_type = g_enum_register_static ("GstAudioscaleMethod",
+        audioscale_methods);
+  }
+  return audioscale_method_type;
+}
 
 static void gst_audioscale_base_init (gpointer g_class);
 static void gst_audioscale_class_init (AudioscaleClass * klass);
@@ -109,8 +110,7 @@ static void gst_audioscale_init (Audioscale * audioscale);
 static void gst_audioscale_dispose (GObject * object);
 
 static void gst_audioscale_chain (GstPad * pad, GstData * _data);
-static GstElementStateReturn
-    gst_audioscale_change_state (GstElement * element);
+static GstElementStateReturn gst_audioscale_change_state (GstElement * element);
 
 static void gst_audioscale_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -123,29 +123,31 @@ static GstElementClass *parent_class = NULL;
 
 /*static guint gst_audioscale_signals[LAST_SIGNAL] = { 0 }; */
 
-GType audioscale_get_type (void)
+GType
+audioscale_get_type (void)
 {
   static GType audioscale_type = 0;
 
-  if (!audioscale_type)
-  {
+  if (!audioscale_type) {
     static const GTypeInfo audioscale_info = {
-    sizeof (AudioscaleClass),
-          gst_audioscale_base_init,
-          NULL,
-          (GClassInitFunc) gst_audioscale_class_init,
-          NULL,
-          NULL,
-          sizeof (Audioscale), 0, (GInstanceInitFunc) gst_audioscale_init,};
+      sizeof (AudioscaleClass),
+      gst_audioscale_base_init,
+      NULL,
+      (GClassInitFunc) gst_audioscale_class_init,
+      NULL,
+      NULL,
+      sizeof (Audioscale), 0, (GInstanceInitFunc) gst_audioscale_init,
+    };
 
     audioscale_type =
-        g_type_register_static (GST_TYPE_ELEMENT, "Audioscale",
+        g_type_register_static (GST_TYPE_ELEMENT, "GstAudioscale",
         &audioscale_info, 0);
   }
   return audioscale_type;
 }
 
-static void gst_audioscale_base_init (gpointer g_class)
+static void
+gst_audioscale_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
 
@@ -157,7 +159,8 @@ static void gst_audioscale_base_init (gpointer g_class)
   gst_element_class_set_details (gstelement_class, &gst_audioscale_details);
 }
 
-static void gst_audioscale_class_init (AudioscaleClass * klass)
+static void
+gst_audioscale_class_init (AudioscaleClass * klass)
 {
   GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
@@ -186,25 +189,26 @@ static void gst_audioscale_class_init (AudioscaleClass * klass)
 }
 
 static GstStaticCaps gst_audioscale_passthru_caps =
-    GST_STATIC_CAPS ("audio/x-raw-int, channels = [ 3, MAX ]");
+    GST_STATIC_CAPS
+    ("audio/x-raw-int, channels = [ 3, MAX ]");
 static GstStaticCaps gst_audioscale_convert_caps =
-    GST_STATIC_CAPS ("audio/x-raw-int, channels = [ 1, 2 ]");
+GST_STATIC_CAPS ("audio/x-raw-int, channels = [ 1, 2 ]");
 
-static GstCaps *gst_audioscale_expand_caps (const GstCaps * caps)
+static GstCaps *
+gst_audioscale_expand_caps (const GstCaps * caps)
 {
   GstCaps *caps1, *caps2;
   int i;
 
-    caps1 = gst_caps_intersect (caps,
+  caps1 = gst_caps_intersect (caps,
       gst_static_caps_get (&gst_audioscale_passthru_caps));
-    caps2 = gst_caps_intersect (caps,
+  caps2 = gst_caps_intersect (caps,
       gst_static_caps_get (&gst_audioscale_convert_caps));
 
-  for (i = 0; i < gst_caps_get_size (caps2); i++)
-  {
+  for (i = 0; i < gst_caps_get_size (caps2); i++) {
     GstStructure *structure = gst_caps_get_structure (caps2, i);
 
-      gst_structure_set (structure, "rate", GST_TYPE_INT_RANGE, 1, G_MAXINT,
+    gst_structure_set (structure, "rate", GST_TYPE_INT_RANGE, 1, G_MAXINT,
         NULL);
   }
 
@@ -213,7 +217,8 @@ static GstCaps *gst_audioscale_expand_caps (const GstCaps * caps)
   return caps1;
 }
 
-static GstCaps *gst_audioscale_getcaps (GstPad * pad)
+static GstCaps *
+gst_audioscale_getcaps (GstPad * pad)
 {
   Audioscale *audioscale;
   GstPad *otherpad;
@@ -231,7 +236,8 @@ static GstCaps *gst_audioscale_getcaps (GstPad * pad)
   return caps;
 }
 
-static GstCaps *gst_audioscale_fixate (GstPad * pad, const GstCaps * caps)
+static GstCaps *
+gst_audioscale_fixate (GstPad * pad, const GstCaps * caps)
 {
   Audioscale *audioscale;
   gst_resample_t *r;
@@ -240,13 +246,12 @@ static GstCaps *gst_audioscale_fixate (GstPad * pad, const GstCaps * caps)
   GstCaps *copy;
   GstStructure *structure;
 
-    audioscale = GST_AUDIOSCALE (gst_pad_get_parent (pad));
-    r = &(audioscale->gst_resample_template);
+  audioscale = GST_AUDIOSCALE (gst_pad_get_parent (pad));
+  r = &(audioscale->gst_resample_template);
   if (pad == audioscale->srcpad) {
     otherpad = audioscale->sinkpad;
     rate = r->i_rate;
-  } else
-  {
+  } else {
     otherpad = audioscale->srcpad;
     rate = r->o_rate;
   }
@@ -263,7 +268,8 @@ static GstCaps *gst_audioscale_fixate (GstPad * pad, const GstCaps * caps)
   return NULL;
 }
 
-static GstPadLinkReturn gst_audioscale_link (GstPad * pad, const GstCaps * caps)
+static GstPadLinkReturn
+gst_audioscale_link (GstPad * pad, const GstCaps * caps)
 {
   Audioscale *audioscale;
   gst_resample_t *r;
@@ -277,15 +283,14 @@ static GstPadLinkReturn gst_audioscale_link (GstPad * pad, const GstCaps * caps)
   GstPad *otherpad;
   GstCaps *copy;
 
-    audioscale = GST_AUDIOSCALE (gst_pad_get_parent (pad));
-    r = &(audioscale->gst_resample_template);
+  audioscale = GST_AUDIOSCALE (gst_pad_get_parent (pad));
+  r = &(audioscale->gst_resample_template);
 
   if (pad == audioscale->srcpad) {
     otherpad = audioscale->sinkpad;
     rate = &r->o_rate;
     otherrate = &r->i_rate;
-  } else
-  {
+  } else {
     otherpad = audioscale->srcpad;
     rate = &r->i_rate;
     otherrate = &r->o_rate;
@@ -381,26 +386,28 @@ static GstPadLinkReturn gst_audioscale_link (GstPad * pad, const GstCaps * caps)
   return link_ret;
 }
 
-static void *gst_audioscale_get_buffer (void *priv, unsigned int size)
+static void *
+gst_audioscale_get_buffer (void *priv, unsigned int size)
 {
   Audioscale *audioscale = priv;
 
-    GST_DEBUG ("size requested: %u irate: %f orate: %f", size,
+  GST_DEBUG ("size requested: %u irate: %f orate: %f", size,
       audioscale->gst_resample->i_rate, audioscale->gst_resample->o_rate);
-    audioscale->outbuf = gst_buffer_new ();
-    GST_BUFFER_SIZE (audioscale->outbuf) = size;
-    GST_BUFFER_DATA (audioscale->outbuf) = g_malloc (size);
-    GST_BUFFER_TIMESTAMP (audioscale->outbuf) =
+  audioscale->outbuf = gst_buffer_new ();
+  GST_BUFFER_SIZE (audioscale->outbuf) = size;
+  GST_BUFFER_DATA (audioscale->outbuf) = g_malloc (size);
+  GST_BUFFER_TIMESTAMP (audioscale->outbuf) =
       audioscale->gst_resample_offset * GST_SECOND /
       audioscale->gst_resample->o_rate;
-    audioscale->gst_resample_offset +=
+  audioscale->gst_resample_offset +=
       size / sizeof (gint16) / audioscale->gst_resample->channels;
 
-    return GST_BUFFER_DATA (audioscale->outbuf);
+  return GST_BUFFER_DATA (audioscale->outbuf);
 }
 
 /* reduces rate by factor of 2 */
-GstBuffer *gst_audioscale_decrease_rate (Audioscale * audioscale,
+GstBuffer *
+gst_audioscale_decrease_rate (Audioscale * audioscale,
     GstBuffer * buf, double outrate, int cur_iteration)
 {
   gint i, j, curoffset;
@@ -437,7 +444,8 @@ GstBuffer *gst_audioscale_decrease_rate (Audioscale * audioscale,
 }
 
 /* increases rate by factor of 2 */
-GstBuffer *gst_audioscale_increase_rate (Audioscale * audioscale,
+GstBuffer *
+gst_audioscale_increase_rate (Audioscale * audioscale,
     GstBuffer * buf, double outrate, int cur_iteration)
 {
   gint i, j, curoffset;
@@ -474,7 +482,8 @@ GstBuffer *gst_audioscale_increase_rate (Audioscale * audioscale,
   return outbuf;
 }
 
-static void gst_audioscale_init (Audioscale * audioscale)
+static void
+gst_audioscale_init (Audioscale * audioscale)
 {
   gst_resample_t *r;
 
@@ -515,9 +524,12 @@ static void gst_audioscale_init (Audioscale * audioscale)
   audioscale->offsets = NULL;
   audioscale->gst_resample_offset = 0;
   audioscale->increase = FALSE;
+
+  GST_FLAG_SET (audioscale, GST_ELEMENT_EVENT_AWARE);
 }
 
-static void gst_audioscale_dispose (GObject * object)
+static void
+gst_audioscale_dispose (GObject * object)
 {
   Audioscale *audioscale = GST_AUDIOSCALE (object);
 
@@ -533,10 +545,12 @@ static void gst_audioscale_dispose (GObject * object)
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
-static void gst_audioscale_chain (GstPad * pad, GstData * _data)
+static void
+gst_audioscale_chain (GstPad * pad, GstData * _data)
 {
   GstBuffer *buf = GST_BUFFER (_data);
   GstBuffer *tempbuf, *tempbuf2;
+  GstClockTime outduration;
 
   Audioscale *audioscale;
   guchar *data;
@@ -549,6 +563,48 @@ static void gst_audioscale_chain (GstPad * pad, GstData * _data)
   g_return_if_fail (buf != NULL);
 
   audioscale = GST_AUDIOSCALE (gst_pad_get_parent (pad));
+
+  if (GST_IS_EVENT (_data)) {
+    GstEvent *e = GST_EVENT (_data);
+
+    switch (GST_EVENT_TYPE (e)) {
+      case GST_EVENT_DISCONTINUOUS:{
+        gint64 new_off = 0;
+
+        if (!audioscale->gst_resample) {
+          GST_LOG ("Discont before negotiation took place - ignoring");
+        } else if (gst_event_discont_get_value (e, GST_FORMAT_TIME, &new_off)) {
+          /* time -> out-sample */
+          new_off = new_off * audioscale->gst_resample->o_rate / GST_SECOND;
+        } else if (gst_event_discont_get_value (e,
+                GST_FORMAT_DEFAULT, &new_off)) {
+          /* in-sample -> out-sample */
+          new_off *= audioscale->gst_resample->o_rate;
+          new_off /= audioscale->gst_resample->i_rate;
+        } else if (gst_event_discont_get_value (e, GST_FORMAT_BYTES, &new_off)) {
+          new_off /= audioscale->gst_resample->channels;
+          new_off /=
+              (audioscale->gst_resample->format == GST_RESAMPLE_S16) ? 2 : 4;
+          new_off *= audioscale->gst_resample->o_rate;
+          new_off /= audioscale->gst_resample->i_rate;
+        } else {
+          /* *sigh* */
+          GST_DEBUG ("Discont without value - ignoring");
+        }
+        audioscale->gst_resample_offset = new_off;
+        /* fall-through */
+      }
+      default:
+        gst_pad_event_default (pad, e);
+        break;
+    }
+    return;
+  } else if (GST_BUFFER_TIMESTAMP_IS_VALID (buf) && audioscale->gst_resample) {
+    /* update time for out-sample */
+    audioscale->gst_resample_offset = GST_BUFFER_TIMESTAMP (buf) *
+        audioscale->gst_resample->o_rate / GST_SECOND;
+  }
+
   if (audioscale->passthru && audioscale->num_iterations == 0) {
     gst_pad_push (audioscale->srcpad, GST_DATA (buf));
     return;
@@ -556,6 +612,7 @@ static void gst_audioscale_chain (GstPad * pad, GstData * _data)
 
   data = GST_BUFFER_DATA (buf);
   size = GST_BUFFER_SIZE (buf);
+  outduration = GST_BUFFER_DURATION (buf);
 
   GST_DEBUG ("gst_audioscale_chain: got buffer of %ld bytes in '%s'\n",
       size, gst_element_get_name (GST_ELEMENT (audioscale)));
@@ -593,7 +650,11 @@ static void gst_audioscale_chain (GstPad * pad, GstData * _data)
     gst_buffer_unref (tempbuf);
     tempbuf = audioscale->outbuf;
   }
-  gst_pad_push (audioscale->srcpad, GST_DATA (tempbuf));
+  GST_BUFFER_DURATION (tempbuf) = outduration;
+  if (GST_BUFFER_SIZE (tempbuf) > 0)
+    gst_pad_push (audioscale->srcpad, GST_DATA (tempbuf));
+  else
+    gst_buffer_unref (tempbuf);
 
 }
 
@@ -614,16 +675,16 @@ gst_audioscale_change_state (GstElement * element)
 }
 
 static void
-    gst_audioscale_set_property (GObject * object, guint prop_id,
+gst_audioscale_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
 {
   Audioscale *src;
   gst_resample_t *r;
 
   /* it's not null if we got it, but it might not be ours */
-    g_return_if_fail (GST_IS_AUDIOSCALE (object));
-    src = GST_AUDIOSCALE (object);
-    r = &(src->gst_resample_template);
+  g_return_if_fail (GST_IS_AUDIOSCALE (object));
+  src = GST_AUDIOSCALE (object);
+  r = &(src->gst_resample_template);
 
   switch (prop_id) {
     case ARG_FILTERLEN:
@@ -631,9 +692,11 @@ static void
       GST_DEBUG_OBJECT (GST_ELEMENT (src), "new filter length %d\n",
           r->filter_length);
       break;
-      case ARG_METHOD:r->method = g_value_get_enum (value);
+    case ARG_METHOD:
+      r->method = g_value_get_enum (value);
       break;
-      default:G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
 
@@ -641,7 +704,7 @@ static void
 }
 
 static void
-    gst_audioscale_get_property (GObject * object, guint prop_id,
+gst_audioscale_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec)
 {
   Audioscale *src;
@@ -664,7 +727,8 @@ static void
 }
 
 
-static gboolean plugin_init (GstPlugin * plugin)
+static gboolean
+plugin_init (GstPlugin * plugin)
 {
   /* load support library */
   if (!gst_library_load ("gstresample"))
