@@ -85,12 +85,10 @@ static void gst_matroska_demux_set_clock (GstElement * element,
 /* caps functions */
 static GstCaps *gst_matroska_demux_video_caps (GstMatroskaTrackVideoContext
     * videocontext,
-    const gchar * codec_id,
-    gpointer data, guint size, GstMatroskaDemux * demux);
+    const gchar * codec_id, gpointer data, guint size, gchar ** codec_name);
 static GstCaps *gst_matroska_demux_audio_caps (GstMatroskaTrackAudioContext
     * audiocontext,
-    const gchar * codec_id,
-    gpointer data, guint size, GstMatroskaDemux * demux);
+    const gchar * codec_id, gpointer data, guint size, gchar ** codec_name);
 static GstCaps *gst_matroska_demux_complex_caps (GstMatroskaTrackComplexContext
     * complexcontext, const gchar * codec_id, gpointer data, guint size);
 static GstCaps
@@ -299,6 +297,8 @@ gst_matroska_demux_add_stream (GstMatroskaDemux * demux)
   gchar *padname = NULL;
   gboolean res = TRUE;
   guint32 id;
+  GstTagList *list = NULL;
+  gchar *codec;
 
   if (demux->num_streams >= GST_MATROSKA_DEMUX_MAX_STREAMS) {
     GST_WARNING ("Maximum number of streams (%d) exceeded, skipping",
@@ -838,7 +838,13 @@ gst_matroska_demux_add_stream (GstMatroskaDemux * demux)
       templ = gst_element_class_get_pad_template (klass, "video_%02d");
       caps = gst_matroska_demux_video_caps (videocontext,
           context->codec_id,
-          context->codec_priv, context->codec_priv_size, demux);
+          context->codec_priv, context->codec_priv_size, &codec);
+      if (codec) {
+        list = gst_tag_list_new ();
+        gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
+            GST_TAG_VIDEO_CODEC, codec, NULL);
+        g_free (codec);
+      }
       break;
     }
 
@@ -849,8 +855,14 @@ gst_matroska_demux_add_stream (GstMatroskaDemux * demux)
       templ = gst_element_class_get_pad_template (klass, "audio_%02d");
       caps = gst_matroska_demux_audio_caps (audiocontext,
           context->codec_id,
-          context->codec_priv, context->codec_priv_size, demux);
+          context->codec_priv, context->codec_priv_size, &codec);
       audiocontext->first_frame = TRUE;
+      if (codec) {
+        list = gst_tag_list_new ();
+        gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
+            GST_TAG_AUDIO_CODEC, codec, NULL);
+        g_free (codec);
+      }
       break;
     }
 
@@ -881,6 +893,13 @@ gst_matroska_demux_add_stream (GstMatroskaDemux * demux)
       g_assert (0);
   }
 
+  if (context->language) {
+    if (!list)
+      list = gst_tag_list_new ();
+    gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
+        GST_TAG_LANGUAGE_CODE, context->language, NULL);
+  }
+
   /* the pad in here */
   context->pad = gst_pad_new_from_template (templ, padname);
   context->caps = caps ? caps : gst_caps_new_empty ();
@@ -902,6 +921,11 @@ gst_matroska_demux_add_stream (GstMatroskaDemux * demux)
 
   gst_element_add_pad (GST_ELEMENT (demux), context->pad);
 
+  /* tags */
+  if (list) {
+    gst_element_found_tags_for_pad (GST_ELEMENT (demux), context->pad, 0, list);
+  }
+
   /* tadaah! */
   return TRUE;
 }
@@ -1260,7 +1284,7 @@ gst_matroska_demux_parse_index (GstMatroskaDemux * demux, gboolean prevent_eos)
           switch (id) {
               /* one single index entry ('point') */
             case GST_MATROSKA_ID_CUETIME:{
-              gint64 time;
+              guint64 time;
 
               if (!gst_ebml_read_uint (ebml, &id, &time)) {
                 res = FALSE;
@@ -1607,21 +1631,47 @@ gst_matroska_demux_parse_metadata (GstMatroskaDemux * demux,
               if (tag && value) {
                 for (i = 0; tag_conv[i].matroska_tagname != NULL; i++) {
                   if (!strcmp (tag_conv[i].matroska_tagname, tag)) {
-                    GValue src = { 0 }
-                    , dest = {
-                    0};
                     const gchar *type = tag_conv[i].gstreamer_tagname;
-                    GType dest_type = gst_tag_get_type (type);
 
-                    g_value_init (&src, G_TYPE_STRING);
-                    g_value_set_string (&src, value);
-                    g_value_init (&dest, dest_type);
-                    g_value_transform (&src, &dest);
-                    g_value_unset (&src);
-                    gst_tag_list_add_values (taglist, GST_TAG_MERGE_APPEND,
-                        type, &dest, NULL);
-                    g_value_unset (&dest);
-                    have_tags = TRUE;
+                    if (strcmp (type, GST_TAG_DATE) == 0) {
+                      GDate *date;
+                      guint y, d = 1, m = 1;
+                      gchar *check = value;
+
+                      y = strtoul (check, &check, 10);
+                      if (*check == '-') {
+                        check++;
+                        m = strtoul (check, &check, 10);
+                        if (*check == '-') {
+                          check++;
+                          d = strtoul (check, &check, 10);
+                        }
+                      }
+                      if (*check != '\0')
+                        break;
+                      if (y == 0)
+                        break;
+                      date = g_date_new_dmy (d, m, y);
+                      y = g_date_get_julian (date);
+                      g_date_free (date);
+                      gst_tag_list_add (taglist, GST_TAG_MERGE_APPEND, type, y,
+                          NULL);
+                      have_tags = TRUE;
+                    } else {
+                      GValue src = { 0, };
+                      GValue dest = { 0, };
+                      GType dest_type = gst_tag_get_type (type);
+
+                      g_value_init (&src, G_TYPE_STRING);
+                      g_value_set_string (&src, value);
+                      g_value_init (&dest, dest_type);
+                      g_value_transform (&src, &dest);
+                      g_value_unset (&src);
+                      gst_tag_list_add_values (taglist, GST_TAG_MERGE_APPEND,
+                          type, &dest, NULL);
+                      g_value_unset (&dest);
+                      have_tags = TRUE;
+                    }
                     break;
                   }
                 }
@@ -1796,7 +1846,8 @@ gst_matroska_demux_parse_blockgroup (GstMatroskaDemux * demux,
   guint64 block_duration = 0;
   GstBuffer *buf = NULL;
   gint stream = 0, n, laces = 0;
-  guint size = 0, *lace_size = NULL;
+  guint size = 0;
+  gint *lace_size = NULL;
   gint64 time = 0;
 
   while (res) {
@@ -2445,7 +2496,7 @@ gst_matroska_demux_loop (GstElement * element)
 
 static GstCaps *
 gst_matroska_demux_video_caps (GstMatroskaTrackVideoContext * videocontext,
-    const gchar * codec_id, gpointer data, guint size, GstMatroskaDemux * demux)
+    const gchar * codec_id, gpointer data, guint size, gchar ** codec_name)
 {
   GstMatroskaTrackContext *context = (GstMatroskaTrackContext *) videocontext;
   GstCaps *caps = NULL;
@@ -2454,9 +2505,6 @@ gst_matroska_demux_video_caps (GstMatroskaTrackVideoContext * videocontext,
     gst_riff_strf_vids *vids = NULL;
 
     if (data) {
-      char *codec_name = NULL;
-      GstTagList *list = gst_tag_list_new ();
-
       vids = (gst_riff_strf_vids *) data;
 
       /* assure size is big enough */
@@ -2484,14 +2532,7 @@ gst_matroska_demux_video_caps (GstMatroskaTrackVideoContext * videocontext,
       vids->imp_colors = GUINT32_FROM_LE (vids->imp_colors);
 
       caps = gst_riff_create_video_caps (vids->compression, NULL, vids,
-          &codec_name);
-      gst_tag_list_add (list, GST_TAG_MERGE_APPEND, GST_TAG_VIDEO_CODEC,
-          codec_name, NULL);
-      if (GST_IS_ELEMENT (demux))
-        gst_element_found_tags (GST_ELEMENT (demux), list);
-      gst_tag_list_free (list);
-      if (codec_name)
-        g_free (codec_name);
+          codec_name);
     } else {
       caps = gst_riff_create_video_template_caps ();
     }
@@ -2502,7 +2543,13 @@ gst_matroska_demux_video_caps (GstMatroskaTrackVideoContext * videocontext,
 
       switch (videocontext->fourcc) {
         case GST_MAKE_FOURCC ('I', '4', '2', '0'):
+          if (codec_name)
+            *codec_name = g_strdup ("Raw planar YUV 4:2:0");
+          fourcc = videocontext->fourcc;
+          break;
         case GST_MAKE_FOURCC ('Y', 'U', 'Y', '2'):
+          if (codec_name)
+            *codec_name = g_strdup ("Raw packed YUV 4:2:2");
           fourcc = videocontext->fourcc;
           break;
 
@@ -2521,6 +2568,8 @@ gst_matroska_demux_video_caps (GstMatroskaTrackVideoContext * videocontext,
   } else if (!strcmp (codec_id, GST_MATROSKA_CODEC_ID_VIDEO_MPEG4_SP)) {
     caps = gst_caps_new_simple ("video/x-divx",
         "divxversion", G_TYPE_INT, 4, NULL);
+    if (codec_name)
+      *codec_name = g_strdup ("MPEG-4 simple profile");
   } else if (!strcmp (codec_id, GST_MATROSKA_CODEC_ID_VIDEO_MPEG4_ASP) ||
       !strcmp (codec_id, GST_MATROSKA_CODEC_ID_VIDEO_MPEG4_AP)) {
     caps = gst_caps_new_full (gst_structure_new ("video/x-divx",
@@ -2529,11 +2578,19 @@ gst_matroska_demux_video_caps (GstMatroskaTrackVideoContext * videocontext,
         gst_structure_new ("video/mpeg",
             "mpegversion", G_TYPE_INT, 4,
             "systemstream", G_TYPE_BOOLEAN, FALSE, NULL), NULL);
+    if (codec_name) {
+      if (!strcmp (codec_id, GST_MATROSKA_CODEC_ID_VIDEO_MPEG4_ASP))
+        *codec_name = g_strdup ("MPEG-4 advanced simple profile");
+      else
+        *codec_name = g_strdup ("MPEG-4 advanced profile");
+    }
   } else if (!strcmp (codec_id, GST_MATROSKA_CODEC_ID_VIDEO_MSMPEG4V3)) {
     caps = gst_caps_new_full (gst_structure_new ("video/x-divx",
             "divxversion", G_TYPE_INT, 3, NULL),
         gst_structure_new ("video/x-msmpeg",
             "msmpegversion", G_TYPE_INT, 43, NULL), NULL);
+    if (codec_name)
+      *codec_name = g_strdup ("Microsoft MPEG-4 v.3");
   } else if (!strcmp (codec_id, GST_MATROSKA_CODEC_ID_VIDEO_MPEG1) ||
       !strcmp (codec_id, GST_MATROSKA_CODEC_ID_VIDEO_MPEG2)) {
     gint mpegversion = -1;
@@ -2548,8 +2605,12 @@ gst_matroska_demux_video_caps (GstMatroskaTrackVideoContext * videocontext,
     caps = gst_caps_new_simple ("video/mpeg",
         "systemstream", G_TYPE_BOOLEAN, FALSE,
         "mpegversion", G_TYPE_INT, mpegversion, NULL);
+    if (codec_name)
+      *codec_name = g_strdup_printf ("MPEG-%d video", mpegversion);
   } else if (!strcmp (codec_id, GST_MATROSKA_CODEC_ID_VIDEO_MJPEG)) {
     caps = gst_caps_new_simple ("video/x-jpeg", NULL);
+    if (codec_name)
+      *codec_name = g_strdup ("Motion-JPEG");
   } else {
     GST_WARNING ("Unknown codec '%s', cannot build Caps", codec_id);
     return NULL;
@@ -2669,7 +2730,7 @@ aac_profile_idx (const gchar * codec_id)
 
 static GstCaps *
 gst_matroska_demux_audio_caps (GstMatroskaTrackAudioContext * audiocontext,
-    const gchar * codec_id, gpointer data, guint size, GstMatroskaDemux * demux)
+    const gchar * codec_id, gpointer data, guint size, gchar ** codec_name)
 {
   GstMatroskaTrackContext *context = (GstMatroskaTrackContext *) audiocontext;
   GstCaps *caps = NULL;
@@ -2690,6 +2751,8 @@ gst_matroska_demux_audio_caps (GstMatroskaTrackAudioContext * audiocontext,
 
     caps = gst_caps_new_simple ("audio/mpeg",
         "mpegversion", G_TYPE_INT, 1, "layer", G_TYPE_INT, layer, NULL);
+    if (codec_name)
+      *codec_name = g_strdup_printf ("MPEG-1 layer %d", layer);
   } else if (!strcmp (codec_id, GST_MATROSKA_CODEC_ID_AUDIO_PCM_INT_BE) ||
       !strcmp (codec_id, GST_MATROSKA_CODEC_ID_AUDIO_PCM_INT_LE)) {
     gint endianness = -1;
@@ -2712,6 +2775,9 @@ gst_matroska_demux_audio_caps (GstMatroskaTrackAudioContext * audiocontext,
           "depth = (int) { 8, 16 }, " "width = (int) { 8, 16 }");
     }
     gst_caps_set_simple (caps, "endianness", G_TYPE_INT, endianness, NULL);
+    if (codec_name && audiocontext)
+      *codec_name = g_strdup_printf ("Raw %d-bits PCM audio",
+          audiocontext->bitdepth);
   } else if (!strcmp (codec_id, GST_MATROSKA_CODEC_ID_AUDIO_PCM_FLOAT)) {
     caps = gst_caps_new_simple ("audio/x-raw-float",
         "endianness", G_TYPE_INT, G_BYTE_ORDER,
@@ -2722,19 +2788,25 @@ gst_matroska_demux_audio_caps (GstMatroskaTrackAudioContext * audiocontext,
     } else {
       gst_caps_set_simple (caps, "width", GST_TYPE_INT_RANGE, 32, 64, NULL);
     }
+    if (codec_name && audiocontext)
+      *codec_name = g_strdup_printf ("Raw %d-bits floating-point audio",
+          audiocontext->bitdepth);
   } else if (!strncmp (codec_id, GST_MATROSKA_CODEC_ID_AUDIO_AC3,
-          strlen (GST_MATROSKA_CODEC_ID_AUDIO_AC3)) ||
-      !strcmp (codec_id, GST_MATROSKA_CODEC_ID_AUDIO_DTS)) {
+          strlen (GST_MATROSKA_CODEC_ID_AUDIO_AC3))) {
     caps = gst_caps_new_simple ("audio/x-ac3", NULL);
+    if (codec_name)
+      *codec_name = g_strdup ("AC-3 audio");
+  } else if (!strcmp (codec_id, GST_MATROSKA_CODEC_ID_AUDIO_DTS)) {
+    caps = gst_caps_new_simple ("audio/x-dts", NULL);
+    if (codec_name)
+      *codec_name = g_strdup ("DTS audio");
   } else if (!strcmp (codec_id, GST_MATROSKA_CODEC_ID_AUDIO_VORBIS)) {
     caps = gst_caps_new_simple ("audio/x-vorbis", NULL);
+    /* vorbis decoder does tags */
   } else if (!strcmp (codec_id, GST_MATROSKA_CODEC_ID_AUDIO_ACM)) {
     gst_riff_strf_auds *auds = NULL;
 
     if (data) {
-      char *codec_name = NULL;
-      GstTagList *list = gst_tag_list_new ();
-
       auds = (gst_riff_strf_auds *) data;
 
       /* little-endian -> byte-order */
@@ -2745,14 +2817,7 @@ gst_matroska_demux_audio_caps (GstMatroskaTrackAudioContext * audiocontext,
       auds->blockalign = GUINT16_FROM_LE (auds->blockalign);
       auds->size = GUINT16_FROM_LE (auds->size);
 
-      caps = gst_riff_create_audio_caps (auds->format, NULL, auds, &codec_name);
-      gst_tag_list_add (list, GST_TAG_MERGE_APPEND, GST_TAG_AUDIO_CODEC,
-          codec_name, NULL);
-      if (GST_IS_ELEMENT (demux))
-        gst_element_found_tags (GST_ELEMENT (demux), list);
-      gst_tag_list_free (list);
-      if (codec_name)
-        g_free (codec_name);
+      caps = gst_riff_create_audio_caps (auds->format, NULL, auds, codec_name);
     } else {
       caps = gst_riff_create_audio_template_caps ();
     }
@@ -2798,10 +2863,13 @@ gst_matroska_demux_audio_caps (GstMatroskaTrackAudioContext * audiocontext,
       g_assert (0);
 
     caps = gst_caps_new_simple ("audio/mpeg",
-        "mpegversion", G_TYPE_INT, mpegversion, NULL);
+        "mpegversion", G_TYPE_INT, mpegversion,
+        "framed", G_TYPE_BOOLEAN, TRUE, NULL);
     if (priv) {
       gst_caps_set_simple (caps, "codec_data", GST_TYPE_BUFFER, priv, NULL);
     }
+    if (codec_name)
+      *codec_name = g_strdup_printf ("MPEG-%d AAC audio", mpegversion);
   } else if (!strcmp (codec_id, GST_MATROSKA_CODEC_ID_AUDIO_TTA)) {
     if (audiocontext != NULL) {
       caps = gst_caps_new_simple ("audio/x-tta",
@@ -2810,6 +2878,8 @@ gst_matroska_demux_audio_caps (GstMatroskaTrackAudioContext * audiocontext,
       caps = gst_caps_from_string ("audio/x-tta, "
           "width = (int) { 8, 16, 24 }");
     }
+    if (codec_name)
+      *codec_name = g_strdup ("TTA audio");
   } else {
     GST_WARNING ("Unknown codec '%s', cannot build Caps", codec_id);
     return NULL;
