@@ -168,7 +168,7 @@ gst_matroska_mux_get_type (void)
 
     gst_matroska_mux_type =
         g_type_register_static (GST_TYPE_EBML_WRITE,
-        "GstMatroskaMmux", &gst_matroska_mux_info, 0);
+        "GstMatroskaMux", &gst_matroska_mux_info, 0);
   }
 
   return gst_matroska_mux_type;
@@ -248,7 +248,7 @@ gst_matroska_mux_init (GstMatroskaMux * mux)
 }
 
 static guint32
-gst_matroska_mux_create_uid ()
+gst_matroska_mux_create_uid (void)
 {
   guint32 uid = 0;
   GRand *rand = g_rand_new ();
@@ -303,6 +303,8 @@ gst_matroska_mux_reset (GstElement * element)
   mux->num_t_streams = 0;
   mux->num_v_streams = 0;
 
+/* FIXME x-gst-metadata : prehistoric way to do metadata in caps. is done via signals actually */
+
   /* reset media info  (to default) */
   gst_caps_replace (&mux->metadata,
       gst_caps_new_simple ("application/x-gst-metadata",
@@ -643,8 +645,8 @@ gst_matroska_mux_request_new_pad (GstElement * element,
 
   pad = gst_pad_new_from_template (templ, name);
   g_free (name);
-  gst_element_add_pad (element, pad);
   gst_pad_set_link_function (pad, linkfunc);
+  gst_element_add_pad (element, pad);
   context->index = mux->num_streams++;
   mux->sink[context->index].track = context;
   context->pad = pad;
@@ -1130,6 +1132,11 @@ gst_matroska_mux_loop (GstElement * element)
   GstMatroskaMux *mux = GST_MATROSKA_MUX (element);
   guint i;
 
+  if (gst_matroska_mux_prepare_data (mux) == -1) {
+    GST_ELEMENT_ERROR (element, STREAM, MUX, (NULL), ("No data"));
+    return;
+  }
+
   /* start with a header */
   if (mux->state == GST_MATROSKA_MUX_STATE_START) {
     if (mux->num_streams == 0) {
