@@ -3,21 +3,37 @@
  *  Copyright (C) 1998-2001 Andy Lo A Foe <andy@alsaplayer.org>
  *  Original code by Tinic Uro
  *
- *  This code is copied from Alsaplayer.
+ *  This code is copied from Alsaplayer. The orginal code was by Tinic Uro and under
+ *  the BSD license without a advertisig clause. Andy Lo A Foe then relicensed the
+ *  code when he used it for Alsaplayer to GPL with Tinic's permission. Richard Boulton
+ *  then took this code and made a GPL plugin out of it.
+ * 
+ *  7th December 2004 Christian Schaller: Richard Boulton and Andy Lo A Foe gave
+ *  permission to relicense their changes under BSD license so we where able to restore the 
+ *  code to Tinic's original BSD license.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * This file is under what is known as the BSD license:
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * Redistribution and use in source and binary forms, with or without modification, i
+ * are permitted provided that the following conditions are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright notice, this list of 
+ * conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright notice, this list 
+ * of conditions and the following disclaimer in the documentation and/or other materials 
+ * provided with the distribution.
+ * 3. The name of the author may not be used to endorse or promote products derived from 
+ * this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
+ * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
+ * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
+ * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY 
+ * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -47,11 +63,11 @@ monoscope_init (guint32 resx, guint32 resy)
   struct monoscope_state *stateptr;
 
   /* I didn't program monoscope to only do 256*128, but it works that way */
-  g_return_val_if_fail (resx == 256, 0);
-  g_return_val_if_fail (resy == 128, 0);
+  g_return_val_if_fail (resx == 256, NULL);
+  g_return_val_if_fail (resy == 128, NULL);
   stateptr = calloc (1, sizeof (struct monoscope_state));
   if (stateptr == 0)
-    return 0;
+    return NULL;
   stateptr->cstate = convolve_init ();
   colors_init (stateptr->colors);
   return stateptr;
