@@ -49,6 +49,7 @@ struct _GstPlayBin
   GstElement *video_sink;
   GstElement *visualisation;
   GstElement *volume_element;
+  GstElement *textoverlay_element;
   gfloat volume;
 
   /* these are the currently active sinks */
@@ -62,6 +63,12 @@ struct _GstPlayBin
 
   /* our cache for the sinks */
   GHashTable *cache;
+
+  /* boolean to see if we're currently switching groups */
+  gboolean group_switch;
+
+  /* font description */
+  gchar *font_desc;
 };
 
 struct _GstPlayBinClass
@@ -77,7 +84,8 @@ enum
   ARG_VIDEO_SINK,
   ARG_VIS_PLUGIN,
   ARG_VOLUME,
-  ARG_FRAME
+  ARG_FRAME,
+  ARG_FONT_DESC
 };
 
 /* signals */
@@ -90,6 +98,7 @@ static void gst_play_bin_class_init (GstPlayBinClass * klass);
 static void gst_play_bin_init (GstPlayBin * play_bin);
 static void gst_play_bin_dispose (GObject * object);
 
+static void group_switch (GstPlayBaseBin * play_base_bin);
 static void setup_sinks (GstPlayBaseBin * play_base_bin);
 static void remove_sinks (GstPlayBin * play_bin);
 
@@ -178,13 +187,18 @@ gst_play_bin_class_init (GstPlayBinClass * klass)
       g_param_spec_object ("vis-plugin", "Vis plugin",
           "the visualization element to use (NULL = none)",
           GST_TYPE_ELEMENT, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (gobject_klass), ARG_VOLUME,
+  g_object_class_install_property (gobject_klass, ARG_VOLUME,
       g_param_spec_double ("volume", "volume", "volume",
           0.0, VOLUME_MAX_DOUBLE, 1.0, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (gobject_klass), ARG_FRAME,
+  g_object_class_install_property (gobject_klass, ARG_FRAME,
       g_param_spec_boxed ("frame", "Frame",
           "The last frame (NULL = no video available)",
           GST_TYPE_BUFFER, G_PARAM_READABLE));
+  g_object_class_install_property (gobject_klass, ARG_FONT_DESC,
+      g_param_spec_string ("subtitle-font-desc",
+          "Subtitle font description",
+          "Pango font description of font "
+          "to be used for subtitle rendering", NULL, G_PARAM_WRITABLE));
 
   gobject_klass->dispose = GST_DEBUG_FUNCPTR (gst_play_bin_dispose);
 
@@ -202,6 +216,7 @@ gst_play_bin_class_init (GstPlayBinClass * klass)
   gstelement_klass->query = GST_DEBUG_FUNCPTR (gst_play_bin_query);
 
   playbasebin_klass->setup_output_pads = setup_sinks;
+  playbasebin_klass->group_switch = group_switch;
 }
 
 static void
@@ -211,12 +226,15 @@ gst_play_bin_init (GstPlayBin * play_bin)
   play_bin->audio_sink = NULL;
   play_bin->visualisation = NULL;
   play_bin->volume_element = NULL;
+  play_bin->textoverlay_element = NULL;
   play_bin->volume = 1.0;
   play_bin->seekables = NULL;
   play_bin->sinks = NULL;
   play_bin->frame = NULL;
+  play_bin->font_desc = NULL;
   play_bin->cache = g_hash_table_new_full (g_str_hash, g_str_equal,
-      NULL, g_object_unref);
+      NULL, (GDestroyNotify) gst_object_unref);
+  play_bin->group_switch = FALSE;
 
   /* no iterate is needed */
   GST_FLAG_SET (play_bin, GST_BIN_SELF_SCHEDULABLE);
@@ -229,14 +247,33 @@ gst_play_bin_dispose (GObject * object)
 
   play_bin = GST_PLAY_BIN (object);
 
-  remove_sinks (play_bin);
-  g_hash_table_destroy (play_bin->cache);
+  if (play_bin->cache != NULL) {
+    remove_sinks (play_bin);
+    g_hash_table_destroy (play_bin->cache);
+    play_bin->cache = NULL;
+  }
+
+  if (play_bin->audio_sink != NULL) {
+    gst_object_unref (GST_OBJECT (play_bin->audio_sink));
+    play_bin->audio_sink = NULL;
+  }
+  if (play_bin->video_sink != NULL) {
+    gst_object_unref (GST_OBJECT (play_bin->video_sink));
+    play_bin->video_sink = NULL;
+  }
+  if (play_bin->visualisation != NULL) {
+    gst_object_unref (GST_OBJECT (play_bin->visualisation));
+    play_bin->visualisation = NULL;
+  }
+  g_free (play_bin->font_desc);
+  play_bin->font_desc = NULL;
 
   if (G_OBJECT_CLASS (parent_class)->dispose) {
     G_OBJECT_CLASS (parent_class)->dispose (object);
   }
 }
 
+
 static void
 gst_play_bin_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -249,26 +286,55 @@ gst_play_bin_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case ARG_VIDEO_SINK:
+      if (play_bin->video_sink != NULL) {
+        gst_object_unref (GST_OBJECT (play_bin->video_sink));
+      }
       play_bin->video_sink = g_value_get_object (value);
+      if (play_bin->video_sink != NULL) {
+        gst_object_ref (GST_OBJECT (play_bin->video_sink));
+        gst_object_sink (GST_OBJECT (play_bin->video_sink));
+      }
       /* when changing the videosink, we just remove the
        * video pipeline from the cache so that it will be 
        * regenerated with the new sink element */
       g_hash_table_remove (play_bin->cache, "vbin");
       break;
     case ARG_AUDIO_SINK:
+      if (play_bin->audio_sink != NULL) {
+        gst_object_unref (GST_OBJECT (play_bin->audio_sink));
+      }
       play_bin->audio_sink = g_value_get_object (value);
+      if (play_bin->audio_sink != NULL) {
+        gst_object_ref (GST_OBJECT (play_bin->audio_sink));
+        gst_object_sink (GST_OBJECT (play_bin->audio_sink));
+      }
       g_hash_table_remove (play_bin->cache, "abin");
       break;
     case ARG_VIS_PLUGIN:
+      if (play_bin->visualisation != NULL) {
+        gst_object_unref (GST_OBJECT (play_bin->visualisation));
+      }
       play_bin->visualisation = g_value_get_object (value);
+      if (play_bin->visualisation != NULL) {
+        gst_object_ref (GST_OBJECT (play_bin->visualisation));
+        gst_object_sink (GST_OBJECT (play_bin->visualisation));
+      }
       break;
     case ARG_VOLUME:
+      play_bin->volume = g_value_get_double (value);
       if (play_bin->volume_element) {
-        play_bin->volume = g_value_get_double (value);
         g_object_set (G_OBJECT (play_bin->volume_element), "volume",
             play_bin->volume, NULL);
       }
       break;
+    case ARG_FONT_DESC:
+      g_free (play_bin->font_desc);
+      play_bin->font_desc = g_strdup (g_value_get_string (value));
+      if (play_bin->textoverlay_element) {
+        g_object_set (G_OBJECT (play_bin->textoverlay_element),
+            "font-desc", g_value_get_string (value), NULL);
+      }
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -347,7 +413,6 @@ gen_video_element (GstPlayBin * play_bin)
   /* first see if we have it in the cache */
   element = g_hash_table_lookup (play_bin->cache, "vbin");
   if (element != NULL) {
-    g_object_ref (G_OBJECT (element));
     /* need to get the video sink element as we need to add it to the
      * list of seekable elements */
     sink = g_hash_table_lookup (play_bin->cache, "video_sink");
@@ -360,11 +425,11 @@ gen_video_element (GstPlayBin * play_bin)
   conv = gst_element_factory_make ("ffmpegcolorspace", "vconv");
   scale = gst_element_factory_make ("videoscale", "vscale");
   if (play_bin->video_sink) {
-    gst_object_ref (GST_OBJECT (play_bin->video_sink));
     sink = play_bin->video_sink;
   } else {
-    sink = gst_element_factory_make ("xvimagesink", "sink");
+    sink = gst_element_factory_make ("autovideosink", "sink");
   }
+  gst_object_ref (GST_OBJECT (sink));
   g_hash_table_insert (play_bin->cache, "video_sink", sink);
 
   gst_bin_add (GST_BIN (element), identity);
@@ -380,8 +445,9 @@ gen_video_element (GstPlayBin * play_bin)
 
   gst_element_set_state (element, GST_STATE_READY);
 
-  /* ref before adding to the cache */
-  g_object_ref (G_OBJECT (element));
+  /* since we're gonna add it to a bin but don't want to lose it,
+   * we keep a reference. */
+  gst_object_ref (GST_OBJECT (element));
   g_hash_table_insert (play_bin->cache, "vbin", element);
 
 done:
@@ -390,6 +456,49 @@ done:
   return element;
 }
 
+/* make an element for playback of video with subtitles embedded.
+ *
+ *  +--------------------------------------------------+
+ *  | tbin                  +-------------+            |
+ *  |          +-----+      | textoverlay |   +------+ |
+ *  |          | csp | +--video_sink      |   | vbin | |
+ * video_sink-sink  src+ +-text_sink     src-sink    | |
+ *  |          +-----+   |  +-------------+   +------+ |
+ * text_sink-------------+                             |
+ *  +--------------------------------------------------+
+ */
+
+static GstElement *
+gen_text_element (GstPlayBin * play_bin)
+{
+  GstElement *element, *csp, *overlay, *vbin;
+
+  overlay = gst_element_factory_make ("textoverlay", "overlay");
+  g_object_set (G_OBJECT (overlay),
+      "halign", "center", "valign", "bottom", NULL);
+  play_bin->textoverlay_element = overlay;
+  if (play_bin->font_desc) {
+    g_object_set (G_OBJECT (play_bin->textoverlay_element),
+        "font-desc", play_bin->font_desc, NULL);
+  }
+  vbin = gen_video_element (play_bin);
+  if (!overlay) {
+    g_warning ("No overlay (pango) element, subtitles disabled");
+    return vbin;
+  }
+  csp = gst_element_factory_make ("ffmpegcolorspace", "subtitlecsp");
+  element = gst_bin_new ("textbin");
+  gst_element_link_many (csp, overlay, vbin, NULL);
+  gst_bin_add_many (GST_BIN (element), csp, overlay, vbin, NULL);
+
+  gst_element_add_ghost_pad (element,
+      gst_element_get_pad (overlay, "text_sink"), "text_sink");
+  gst_element_add_ghost_pad (element,
+      gst_element_get_pad (csp, "sink"), "sink");
+
+  return element;
+}
+
 /* make the element (bin) that contains the elements needed to perform
  * audio playback. 
  *
@@ -414,7 +523,6 @@ gen_audio_element (GstPlayBin * play_bin)
 
   element = g_hash_table_lookup (play_bin->cache, "abin");
   if (element != NULL) {
-    g_object_ref (G_OBJECT (element));
     sink = g_hash_table_lookup (play_bin->cache, "audio_sink");
     goto done;
   }
@@ -427,11 +535,13 @@ gen_audio_element (GstPlayBin * play_bin)
   play_bin->volume_element = volume;
 
   if (play_bin->audio_sink) {
-    gst_object_ref (GST_OBJECT (play_bin->audio_sink));
     sink = play_bin->audio_sink;
   } else {
-    sink = gst_element_factory_make ("osssink", "sink");
+    sink = gst_element_factory_make ("autoaudiosink", "sink");
+    play_bin->audio_sink = GST_ELEMENT (gst_object_ref (GST_OBJECT (sink)));
   }
+
+  gst_object_ref (GST_OBJECT (sink));
   g_hash_table_insert (play_bin->cache, "audio_sink", sink);
 
   gst_bin_add (GST_BIN (element), conv);
@@ -448,8 +558,9 @@ gen_audio_element (GstPlayBin * play_bin)
 
   gst_element_set_state (element, GST_STATE_READY);
 
-  /* ref before adding to the cache */
-  g_object_ref (G_OBJECT (element));
+  /* since we're gonna add it to a bin but don't want to lose it,
+   * we keep a reference. */
+  gst_object_ref (GST_OBJECT (element));
   g_hash_table_insert (play_bin->cache, "abin", element);
 
 done:
@@ -479,7 +590,7 @@ done:
  *  |   |  |      |                                      +-------------------+ |
  *  |   |  +------+                                                            |
  * sink-+                                                                      |
- *  +--------------------------------------------------------------------------+
+   +--------------------------------------------------------------------------+
  */
 static GstElement *
 gen_vis_element (GstPlayBin * play_bin)
@@ -536,6 +647,13 @@ gen_vis_element (GstPlayBin * play_bin)
   return element;
 }
 
+/* set for group switch */
+static void
+group_switch (GstPlayBaseBin * play_base_bin)
+{
+  GST_PLAY_BIN (play_base_bin)->group_switch = TRUE;
+}
+
 /* get rid of all installed sinks */
 static void
 remove_sinks (GstPlayBin * play_bin)
@@ -545,6 +663,7 @@ remove_sinks (GstPlayBin * play_bin)
   GstElement *element;
 
   GST_DEBUG ("removesinks");
+  play_bin->group_switch = FALSE;
   element = g_hash_table_lookup (play_bin->cache, "abin");
   if (element != NULL) {
     parent = gst_element_get_parent (element);
@@ -565,8 +684,11 @@ remove_sinks (GstPlayBin * play_bin)
 
   for (sinks = play_bin->sinks; sinks; sinks = g_list_next (sinks)) {
     GstElement *element = GST_ELEMENT (sinks->data);
+    GstPad *pad = gst_element_get_pad (element, "sink");
 
     GST_LOG ("removing sink %p", element);
+    if (GST_PAD_PEER (pad))
+      gst_pad_unlink (GST_PAD_PEER (pad), pad);
     gst_bin_remove (GST_BIN (play_bin), element);
   }
   g_list_free (play_bin->sinks);
@@ -578,6 +700,8 @@ remove_sinks (GstPlayBin * play_bin)
     gst_buffer_unref (play_bin->frame);
     play_bin->frame = NULL;
   }
+
+  play_bin->textoverlay_element = NULL;
 }
 
 /* loop over the streams and set up the pipeline to play this
@@ -589,66 +713,70 @@ remove_sinks (GstPlayBin * play_bin)
  * this should eventually be handled with a tuner interface so that
  * one can switch the streams.
  */
+static gboolean
+add_sink (GstPlayBin * play_bin, GstElement * sink, GstPad * srcpad)
+{
+  GstPad *sinkpad;
+  gboolean res;
+
+  /* we found a sink for this stream, now try to install it */
+  sinkpad = gst_element_get_pad (sink, "sink");
+  res = gst_pad_link (srcpad, sinkpad);
+  GST_DEBUG ("Adding sink with state %d (parent: %d, peer: %d)\n",
+      GST_STATE (sink), GST_STATE (play_bin),
+      GST_STATE (gst_pad_get_parent (srcpad)));
+
+  /* try to link the pad of the sink to the stream */
+  if (!res) {
+    gchar *capsstr;
+
+    /* could not link this stream */
+    capsstr = gst_caps_to_string (gst_pad_get_caps (srcpad));
+    g_warning ("could not link %s", capsstr);
+    g_free (capsstr);
+  } else {
+    /* we got the sink succesfully linked, now keep the sink
+     * in out internal list */
+    play_bin->sinks = g_list_prepend (play_bin->sinks, sink);
+    gst_bin_add (GST_BIN (play_bin), sink);
+  }
+
+  return res;
+}
+
 static void
 setup_sinks (GstPlayBaseBin * play_base_bin)
 {
   GstPlayBin *play_bin = GST_PLAY_BIN (play_base_bin);
-  GList *streaminfo;
-  GList *s;
-  gint num_audio = 0;
-  gint num_video = 0;
+  GstPlayBaseGroup *group;
+  GList *streaminfo = NULL, *s;
   gboolean need_vis = FALSE;
+  gboolean need_text = FALSE;
+  GstPad *textsrcpad = NULL, *textsinkpad = NULL;
+  GstElement *sink;
 
-  /* FIXME: do this nicer, like taking a look at the installed
-   * bins and figuring out if we can simply reconnect them, remove
-   * or add them. */
-  if (GST_STATE (play_base_bin) == GST_STATE_PLAYING) {
+  /* get rid of existing sinks */
+  if (play_bin->sinks) {
     remove_sinks (play_bin);
   }
   GST_DEBUG ("setupsinks");
-  /* get info about the stream */
-  g_object_get (G_OBJECT (play_bin), "stream-info", &streaminfo, NULL);
 
-  /* first examine the streams we have */
-  for (s = streaminfo; s; s = g_list_next (s)) {
-    GObject *obj = G_OBJECT (s->data);
-    gint type;
-    GstPad *srcpad;
-
-    g_object_get (obj, "type", &type, NULL);
-    /* we don't care about streams with their own sink */
-    if (type == 4)
-      continue;
-
-    /* else we need to get the pad */
-    g_object_get (obj, "object", &srcpad, NULL);
-
-    /* hmm.. pad is allready linked */
-    if (gst_pad_is_linked (srcpad))
-      continue;
-
-    if (type == 1) {
-      num_audio++;
-    } else if (type == 2) {
-      num_video++;
-    }
-  }
-  /* no video, use vis */
-  if (num_video == 0 && num_audio > 0 && play_bin->visualisation) {
+  /* find out what to do */
+  group = play_base_bin->queued_groups->data;
+  if (group->type[GST_STREAM_TYPE_VIDEO - 1].npads > 0 &&
+      group->type[GST_STREAM_TYPE_TEXT - 1].npads > 0) {
+    need_text = TRUE;
+  } else if (group->type[GST_STREAM_TYPE_VIDEO - 1].npads == 0 &&
+      group->type[GST_STREAM_TYPE_AUDIO - 1].npads > 0 &&
+      play_bin->visualisation != NULL) {
     need_vis = TRUE;
   }
 
-  num_audio = 0;
-  num_video = 0;
-
   /* now actually connect everything */
+  g_object_get (G_OBJECT (play_base_bin), "stream-info", &streaminfo, NULL);
   for (s = streaminfo; s; s = g_list_next (s)) {
     GObject *obj = G_OBJECT (s->data);
     gint type;
-    GstPad *srcpad, *sinkpad;
-    GstElement *sink = NULL;
-    gboolean res;
-    gboolean mute = FALSE;
     GstObject *object;
 
     g_object_get (obj, "type", &type, NULL);
@@ -657,73 +785,39 @@ setup_sinks (GstPlayBaseBin * play_base_bin)
     /* use the sink elements as seek entry point */
     if (type == 4) {
       play_bin->seekables = g_list_prepend (play_bin->seekables, object);
-      continue;
     }
+  }
 
-    srcpad = GST_PAD (object);
-
-    /* pas is allready linked, go to the next pad */
-    if (gst_pad_is_linked (srcpad))
-      continue;
-
-    if (type == 1) {
-      if (num_audio > 0) {
-        g_warning ("two audio streams found, playing first one");
-        mute = TRUE;
-      } else {
-        if (need_vis) {
-          sink = gen_vis_element (play_bin);
-        } else {
-          sink = gen_audio_element (play_bin);
-        }
-        num_audio++;
-      }
-    } else if (type == 2) {
-      if (num_video > 0) {
-        g_warning ("two video streams found, playing first one");
-        mute = TRUE;
-      } else {
-        sink = gen_video_element (play_bin);
-        num_video++;
-      }
-    } else if (type == 4) {
-      /* we can ignore these streams here */
+  /* link audio */
+  if (group->type[GST_STREAM_TYPE_AUDIO - 1].npads > 0) {
+    if (need_vis) {
+      sink = gen_vis_element (play_bin);
     } else {
-      g_warning ("unknown stream found");
-      mute = TRUE;
+      sink = gen_audio_element (play_bin);
     }
+    add_sink (play_bin, sink,
+        gst_element_get_pad (group->type[GST_STREAM_TYPE_AUDIO - 1].preroll,
+            "src"));
+  }
 
-    /* we found a sink for this stream, now try to install it */
-    if (sink != NULL) {
-      gst_object_ref (GST_OBJECT (sink));
-      gst_bin_add (GST_BIN (play_bin), sink);
-      GST_DEBUG ("Adding sink with state %d (parent: %d, peer: %d)\n",
-          GST_STATE (sink), GST_STATE (play_bin),
-          GST_STATE (gst_pad_get_parent (srcpad)));
-      sinkpad = gst_element_get_pad (sink, "sink");
-      /* try to link the pad of the sink to the stream */
-      res = gst_pad_link (srcpad, sinkpad);
-      if (!res) {
-        gchar *capsstr;
+  /* link video */
+  if (group->type[GST_STREAM_TYPE_VIDEO - 1].npads > 0) {
+    if (need_text) {
+      sink = gen_text_element (play_bin);
 
-        /* could not link this stream */
-        capsstr = gst_caps_to_string (gst_pad_get_caps (srcpad));
-        g_warning ("could not link %s", capsstr);
-        g_free (capsstr);
-        GST_LOG ("removing sink %p", sink);
-        gst_bin_remove (GST_BIN (play_bin), sink);
-        mute = TRUE;
-      } else {
-        /* we got the sink succesfully linked, now keep the sink
-         * in out internal list */
-        play_bin->sinks = g_list_prepend (play_bin->sinks, sink);
+      textsinkpad = gst_element_get_pad (sink, "text_sink");
+      textsrcpad =
+          gst_element_get_pad (group->type[GST_STREAM_TYPE_TEXT - 1].preroll,
+          "src");
+      if (textsinkpad && textsrcpad) {
+        gst_pad_link (textsrcpad, textsinkpad);
       }
+    } else {
+      sink = gen_video_element (play_bin);
     }
-    if (mute) {
-      /* no sink found/needed for this stream. We mute the stream
-       * so that it does not take any resources */
-      g_object_set (G_OBJECT (obj), "mute", TRUE, NULL);
-    }
+    add_sink (play_bin, sink,
+        gst_element_get_pad (group->type[GST_STREAM_TYPE_VIDEO - 1].preroll,
+            "src"));
   }
 }
 
@@ -743,18 +837,24 @@ gst_play_bin_change_state (GstElement * element)
     return ret;
 
   switch (transition) {
-    case GST_STATE_NULL_TO_READY:
-      break;
-    case GST_STATE_READY_TO_PAUSED:
-      break;
-    case GST_STATE_PAUSED_TO_PLAYING:
-      break;
     case GST_STATE_PLAYING_TO_PAUSED:
+      /* Set audio sink state to NULL to release the sound device,
+       * but only if we own it (else we might be in chain-transition). */
+      if (play_bin->audio_sink != NULL && !play_bin->group_switch &&
+          GST_STATE (play_bin->audio_sink) == GST_STATE_PAUSED) {
+        gst_element_set_state (play_bin->audio_sink, GST_STATE_NULL);
+      }
       break;
     case GST_STATE_PAUSED_TO_READY:
-      remove_sinks (play_bin);
-      break;
-    case GST_STATE_READY_TO_NULL:
+      /* Check for NULL because the state transition may be done by
+       * gst_bin_dispose which is called by gst_play_bin_dispose, and in that
+       * case, we don't want to run remove_sinks.
+       * FIXME: should the NULL test be done in remove_sinks? Should we just
+       * set the state to NULL in gst_play_bin_dispose?
+       */
+      if (play_bin->cache != NULL) {
+        remove_sinks (play_bin);
+      }
       break;
     default:
       break;
