@@ -238,25 +238,71 @@ tta_type_find (GstTypeFind * tf, gpointer unused)
 static GstStaticCaps aac_caps = GST_STATIC_CAPS ("audio/mpeg, "
     "mpegversion = (int) { 2, 4 }, framed = (bool) false");
 #define AAC_CAPS (gst_static_caps_get(&aac_caps))
+#define AAC_AMOUNT (4096)
 static void
 aac_type_find (GstTypeFind * tf, gpointer unused)
 {
-  guint8 *data = gst_type_find_peek (tf, 0, 2);
+  guint8 *data = gst_type_find_peek (tf, 0, AAC_AMOUNT);
+  gint snc;
 
-  /* detect adts header
-   * note that this is a pretty lame typefind method (14 bits, 0.006%), so
-   * we'll only use LIKELY
+  /* detect adts header or adif header.
+   * The ADIF header is 4 bytes, that should be OK. The ADTS header, on
+   * the other hand, is 14 bits only, so we require one valid frame with
+   * again a valid syncpoint on the next one (28 bits) for certainty. We
+   * require 4 kB, which is quite a lot, since frames are generally 200-400
+   * bytes.
    */
   if (data) {
-    if (data[0] == 0xFF && (data[1] & 0xF6) == 0xF0) {
-      gboolean mpegversion = (data[1] & 0x08) ? 2 : 4;
-      GstCaps *caps = gst_caps_new_simple ("audio/mpeg",
-          "framed", G_TYPE_BOOLEAN, FALSE,
-          "mpegversion", G_TYPE_INT, mpegversion,
-          NULL);
+    gint n;
 
-      gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, caps);
-      gst_caps_free (caps);
+    for (n = 0; n < AAC_AMOUNT - 3; n++) {
+      snc = GST_READ_UINT16_BE (&data[n]);
+      if ((snc & 0xfff6) == 0xfff0) {
+        /* ADTS header - find frame length */
+        gint len;
+
+        GST_DEBUG ("Found one ADTS syncpoint at offset 0x%x, tracing next...",
+            n);
+        if (AAC_AMOUNT - n < 5) {
+          GST_DEBUG ("Not enough data to parse ADTS header");
+          break;
+        }
+        len = ((data[n + 3] & 0x03) << 11) |
+            (data[n + 4] << 3) | ((data[n + 5] & 0xe0) >> 5);
+        if (n + len + 2 >= AAC_AMOUNT) {
+          GST_DEBUG ("Next frame is not within reach");
+          break;
+        } else if (len == 0) {
+          continue;
+        }
+
+        snc = GST_READ_UINT16_BE (&data[n + len]);
+        if ((snc & 0xfff6) == 0xfff0) {
+          gint mpegversion = (data[n + 1] & 0x08) ? 2 : 4;
+          GstCaps *caps = gst_caps_new_simple ("audio/mpeg",
+              "framed", G_TYPE_BOOLEAN, FALSE,
+              "mpegversion", G_TYPE_INT, mpegversion,
+              NULL);
+
+          gst_type_find_suggest (tf, GST_TYPE_FIND_LIKELY, caps);
+          gst_caps_free (caps);
+
+          GST_DEBUG ("Found ADTS-%d syncpoint at offset 0x%x (framelen %u)",
+              mpegversion, n, len);
+          break;
+        }
+
+        GST_DEBUG ("No next frame found... (should be at 0x%x)", n + len);
+      } else if (!memcmp (&data[n], "ADIF", 4)) {
+        /* ADIF header */
+        GstCaps *caps = gst_caps_new_simple ("audio/mpeg",
+            "framed", G_TYPE_BOOLEAN, FALSE,
+            "mpegversion", G_TYPE_INT, 4,
+            NULL);
+
+        gst_type_find_suggest (tf, GST_TYPE_FIND_LIKELY, caps);
+        gst_caps_free (caps);
+      }
     }
   }
 }
@@ -301,7 +347,8 @@ static guint mp3types_freqs[3][3] = { {11025, 12000, 8000},
 
 static inline guint
 mp3_type_frame_length_from_header (guint32 header, guint * put_layer,
-    guint * put_channels, guint * put_bitrate, guint * put_samplerate)
+    guint * put_channels, guint * put_bitrate, guint * put_samplerate,
+    gboolean * may_be_free_format, gint possible_free_framelen)
 {
   guint bitrate, layer, length, mode, samplerate, version, channels;
 
@@ -328,7 +375,11 @@ mp3_type_frame_length_from_header (guint32 header, guint * put_layer,
 
   /* bitrate index */
   bitrate = header & 0xF;
-  if (bitrate == 15 || bitrate == 0)
+  if (bitrate == 0 && possible_free_framelen == -1) {
+    GST_LOG ("Possibly a free format mp3 - signalling");
+    *may_be_free_format = TRUE;
+  }
+  if (bitrate == 15 || (bitrate == 0 && possible_free_framelen == -1))
     return 0;
 
   /* ignore error correction, too */
@@ -347,15 +398,26 @@ mp3_type_frame_length_from_header (guint32 header, guint * put_layer,
 
   /* lookup */
   channels = (mode == 3) ? 1 : 2;
-  bitrate = mp3types_bitrates[version == 3 ? 0 : 1][layer - 1][bitrate];
   samplerate = mp3types_freqs[version > 0 ? version - 1 : 0][samplerate];
-
-  /* calculating */
-  if (layer == 1) {
-    length = ((12000 * bitrate / samplerate) + length) * 4;
+  if (bitrate == 0) {
+    if (layer == 1) {
+      length *= 4;
+      length += possible_free_framelen;
+      bitrate = length * samplerate / 48000;
+    } else {
+      length += possible_free_framelen;
+      bitrate = length * samplerate /
+          ((layer == 3 && version != 3) ? 72000 : 144000);
+    }
   } else {
-    length += ((layer == 3
-            && version != 3) ? 72000 : 144000) * bitrate / samplerate;
+    /* calculating */
+    bitrate = mp3types_bitrates[version == 3 ? 0 : 1][layer - 1][bitrate];
+    if (layer == 1) {
+      length = ((12000 * bitrate / samplerate) + length) * 4;
+    } else {
+      length += ((layer == 3
+              && version != 3) ? 72000 : 144000) * bitrate / samplerate;
+    }
   }
 
   GST_LOG ("mp3typefind: calculated mp3 frame length of %u bytes", length);
@@ -401,6 +463,7 @@ mp3_type_find (GstTypeFind * tf, gpointer unused)
   guint64 skipped;
 
   for (try = 0; try < 2; try++) {
+    gint last_free_offset = -1, last_free_framelen = -1;
     guint64 start_off = (try == 0) ? 0 : length / 2;
 
     if (try != 0 && start_off == 0)
@@ -420,7 +483,7 @@ mp3_type_find (GstTypeFind * tf, gpointer unused)
       }
       if (*data == 0xFF) {
         guint8 *head_data = NULL;
-        guint layer, bitrate, samplerate, channels;
+        guint layer = 0, bitrate, samplerate, channels;
         guint found = 0;        /* number of valid headers found */
         guint64 offset = skipped;
 
@@ -429,6 +492,7 @@ mp3_type_find (GstTypeFind * tf, gpointer unused)
           guint length;
           guint prev_layer = 0, prev_bitrate = 0,
               prev_channels = 0, prev_samplerate = 0;
+          gboolean free = FALSE;
 
           if (offset + 4 <= skipped + size) {
             head_data = data + offset - skipped;
@@ -439,10 +503,22 @@ mp3_type_find (GstTypeFind * tf, gpointer unused)
             break;
           head = GST_READ_UINT32_BE (head_data);
           if (!(length = mp3_type_frame_length_from_header (head, &layer,
-                      &channels, &bitrate, &samplerate))) {
+                      &channels, &bitrate, &samplerate, &free,
+                      last_free_framelen))) {
+            if (free) {
+              if (last_free_offset == -1)
+                last_free_offset = offset;
+              else {
+                last_free_framelen = offset - last_free_offset;
+                offset = last_free_offset;
+                continue;
+              }
+            } else {
+              last_free_framelen = -1;
+            }
             GST_LOG ("%d. header at offset %" G_GUINT64_FORMAT
-                " (0x%X) was not an mp3 header", found + 1, offset,
-                (guint) offset);
+                " (0x%X) was not an mp3 header (possibly-free: %s)",
+                found + 1, offset, (guint) offset, free ? "yes" : "no");
             break;
           }
           if ((prev_layer && prev_layer != layer) ||
@@ -527,6 +603,58 @@ ac3_type_find (GstTypeFind * tf, gpointer unused)
   }
 }
 
+/*** wavpack *****************************************************************/
+static GstStaticCaps wavpack_caps = GST_STATIC_CAPS ("audio/x-wavpack");
+
+#define WAVPACK_CAPS (gst_static_caps_get(&wavpack_caps))
+
+static void
+wavpack_type_find (GstTypeFind * tf, gpointer unused)
+{
+  guint8 *data = gst_type_find_peek (tf, 0, 32);
+
+  if (!data)
+    return;
+
+  if (data[0] == 'w' && data[1] == 'v' && data[2] == 'p' && data[3] == 'k') {
+    guint32 blocksize, sublen;
+    gint32 left;
+
+    blocksize = GST_READ_UINT32_LE (data + 4);
+    /* peek from offset 0, otherwise it won't work with apedemux */
+    data = gst_type_find_peek (tf, 0, 32 + blocksize);
+    if (!data)
+      return;
+    data += 32;
+    left = (gint32) blocksize;
+    while (left > 2) {
+      sublen = ((guint32) data[1]) << 1;
+      if (data[0] & 0x80) {
+        sublen |= (((guint32) data[2]) << 9) | (((guint32) data[3]) << 17);
+        sublen += 1 + 3;        /* id + length */
+      } else {
+        sublen += 1 + 1;        /* id + length */
+      }
+      if (sublen > blocksize)
+        return;
+      if ((data[0] & 0x20) == 0) {
+        switch (data[0] & 0x0f) {
+          case 0xa:            /* ID_WV_BITSTREAM  */
+          case 0xc:            /* ID_WVX_BITSTREAM */
+            gst_type_find_suggest (tf, GST_TYPE_FIND_LIKELY, WAVPACK_CAPS);
+            return;
+          case 0xb:            /* ID_WVC_BITSTREAM (correction data, do not identify as wavpack) */
+            return;
+          default:
+            break;
+        }
+      }
+      left -= sublen;
+      data += sublen;
+    }
+  }
+}
+
 /*** video/mpeg systemstream **************************************************/
 
 static GstStaticCaps mpeg_sys_caps = GST_STATIC_CAPS ("video/mpeg, "
@@ -846,14 +974,14 @@ qt_type_find (GstTypeFind * tf, gpointer unused)
   guint64 size;
 
   while ((data = gst_type_find_peek (tf, offset, 8)) != NULL) {
-    if (strncmp (&data[4], "wide", 4) != 0 &&
-        strncmp (&data[4], "moov", 4) != 0 &&
-        strncmp (&data[4], "mdat", 4) != 0 &&
-        strncmp (&data[4], "pnot", 4) != 0 &&
-        strncmp (&data[4], "PICT", 4) != 0 &&
-        strncmp (&data[4], "ftyp", 4) != 0 &&
-        strncmp (&data[4], "free", 4) != 0 &&
-        strncmp (&data[4], "skip", 4) != 0) {
+    if (strncmp ((const char *) &data[4], "wide", 4) != 0 &&
+        strncmp ((const char *) &data[4], "moov", 4) != 0 &&
+        strncmp ((const char *) &data[4], "mdat", 4) != 0 &&
+        strncmp ((const char *) &data[4], "pnot", 4) != 0 &&
+        strncmp ((const char *) &data[4], "PICT", 4) != 0 &&
+        strncmp ((const char *) &data[4], "ftyp", 4) != 0 &&
+        strncmp ((const char *) &data[4], "free", 4) != 0 &&
+        strncmp ((const char *) &data[4], "skip", 4) != 0) {
       tip = 0;
       break;
     }
@@ -1555,7 +1683,7 @@ cmml_type_find (GstTypeFind * tf, gpointer private)
 /*** generic typefind for streams that have some data at a specific position***/
 typedef struct
 {
-  guint8 *data;
+  const guint8 *data;
   guint size;
   guint probability;
   GstCaps *caps;
@@ -1579,7 +1707,7 @@ start_with_type_find (GstTypeFind * tf, gpointer private)
 #define TYPE_FIND_REGISTER_START_WITH(plugin,name,rank,ext,_data,_size,_probability)\
 G_BEGIN_DECLS{									\
   GstTypeFindData *sw_data = g_new (GstTypeFindData, 1);			\
-  sw_data->data = _data;						      	\
+  sw_data->data = (const guint8*) _data;				      	\
   sw_data->size = _size;						      	\
   sw_data->probability = _probability;						\
   sw_data->caps = gst_caps_new_simple (name, NULL);			\
@@ -1605,7 +1733,7 @@ riff_type_find (GstTypeFind * tf, gpointer private)
 #define TYPE_FIND_REGISTER_RIFF(plugin,name,rank,ext,_data)			\
 G_BEGIN_DECLS{									\
   GstTypeFindData *sw_data = g_new (GstTypeFindData, 1);			\
-  sw_data->data = _data;						      	\
+  sw_data->data = (const guint8*) _data;				      	\
   sw_data->size = 4;							      	\
   sw_data->probability = GST_TYPE_FIND_MAXIMUM;					\
   sw_data->caps = gst_caps_new_simple (name, NULL);			\
@@ -1682,6 +1810,8 @@ plugin_init (GstPlugin * plugin)
   static gchar *m4a_exts[] = { "m4a", NULL };
   static gchar *q3gp_exts[] = { "3gp", NULL };
   static gchar *aac_exts[] = { "aac", NULL };
+  static gchar *spc_exts[] = { "spc", NULL };
+  static gchar *wavpack_exts[] = { "wv", "wvp", NULL };
 
   GST_DEBUG_CATEGORY_INIT (type_find_debug, "typefindfunctions",
       GST_DEBUG_FG_GREEN | GST_DEBUG_BG_RED, "generic type find functions");
@@ -1828,6 +1958,10 @@ plugin_init (GstPlugin * plugin)
       GST_RANK_MARGINAL, NULL, "\177ELF", 4, GST_TYPE_FIND_MAXIMUM);
   TYPE_FIND_REGISTER (plugin, "adts_mpeg_stream", GST_RANK_SECONDARY,
       aac_type_find, aac_exts, AAC_CAPS, NULL);
+  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-spc", GST_RANK_SECONDARY,
+      spc_exts, "SNES-SPC700 Sound File Data", 27, GST_TYPE_FIND_MAXIMUM);
+  TYPE_FIND_REGISTER (plugin, "audio/x-wavpack", GST_RANK_SECONDARY,
+      wavpack_type_find, wavpack_exts, WAVPACK_CAPS, NULL);
 
   return TRUE;
 }
