@@ -28,10 +28,9 @@ struct vts_color_struct {
 typedef struct paintinfo_struct paintinfo;
 struct paintinfo_struct
 {
-  unsigned char *dest;		/* pointer to first byte of video data */
-  unsigned char *yp, *up, *vp;  /* pointers to first byte of each component
+  guint8 *yp, *up, *vp;  /* pointers to first byte of each component
 				 * for both packed/planar YUV and RGB */
-  unsigned char *endptr;	/* pointer to byte beyond last video data */
+  guint8 *endptr;	/* pointer to byte beyond last video data */
   int ystride;
   int ustride;
   int vstride;
@@ -43,10 +42,10 @@ struct paintinfo_struct
 
 struct fourcc_list_struct
 {
-  char *fourcc;
-  char *name;
+  const char *fourcc;
+  const char *name;
   int bitspp;
-  void (*paint_setup) (paintinfo * p, char *dest);
+  void (*paint_setup) (paintinfo * p, guint8 *dest);
   void (*paint_hline) (paintinfo * p, int x, int y, int w);
   int ext_caps;
   int depth;
@@ -65,11 +64,11 @@ GstStructure *
 	paint_get_structure		(struct fourcc_list_struct *format);
 int	gst_videotestsrc_get_size	(GstVideotestsrc * v, int w, int h);
 void	gst_videotestsrc_smpte		(GstVideotestsrc * v,
-					 unsigned char *dest, int w, int h);
+					 guint8 *dest, int w, int h);
 void	gst_videotestsrc_snow		(GstVideotestsrc * v,
-					 unsigned char *dest, int w, int h);
+					 guint8 *dest, int w, int h);
 void	gst_videotestsrc_black		(GstVideotestsrc * v,
-					 unsigned char *dest, int w, int h);
+					 guint8 *dest, int w, int h);
 
 extern struct fourcc_list_struct fourcc_list[];
 extern int n_fourccs;
