@@ -49,7 +49,8 @@ MotifWmHints, MwmHints;
 static void gst_xvimagesink_buffer_free (GstBuffer * buffer);
 static void gst_xvimagesink_xvimage_destroy (GstXvImageSink * xvimagesink,
     GstXvImage * xvimage);
-
+static void
+gst_xvimagesink_send_pending_navigation (GstXvImageSink * xvimagesink);
 
 /* ElementFactory information */
 static GstElementDetails gst_xvimagesink_details =
@@ -146,7 +147,7 @@ gst_xvimagesink_check_xshm_calls (GstXContext * xcontext)
     goto beach;
   }
 
-  xvimage->SHMInfo.shmaddr = shmat (xvimage->SHMInfo.shmid, 0, 0);
+  xvimage->SHMInfo.shmaddr = shmat (xvimage->SHMInfo.shmid, NULL, 0);
   if (xvimage->SHMInfo.shmaddr == ((void *) -1)) {
     GST_WARNING ("Failed to shmat: %s", g_strerror (errno));
     goto beach;
@@ -166,7 +167,7 @@ gst_xvimagesink_check_xshm_calls (GstXContext * xcontext)
   XSync (xcontext->disp, FALSE);
 
   shmdt (xvimage->SHMInfo.shmaddr);
-  shmctl (xvimage->SHMInfo.shmid, IPC_RMID, 0);
+  shmctl (xvimage->SHMInfo.shmid, IPC_RMID, NULL);
 
   /* To be sure, reset the SHMInfo entry */
   xvimage->SHMInfo.shmaddr = ((void *) -1);
@@ -194,7 +195,7 @@ gst_xvimagesink_xvimage_new (GstXvImageSink * xvimagesink,
   gboolean succeeded = FALSE;
 
   g_return_val_if_fail (GST_IS_XVIMAGESINK (xvimagesink), NULL);
-  GST_DEBUG_OBJECT (xvimagesink, "creating %dx%d", width, height);
+  GST_LOG_OBJECT (xvimagesink, "creating %dx%d", width, height);
 
   xvimage = g_new0 (GstXvImage, 1);
 
@@ -219,7 +220,7 @@ gst_xvimagesink_xvimage_new (GstXvImageSink * xvimagesink,
 
     /* we have to use the returned data_size for our shm size */
     xvimage->size = xvimage->xvimage->data_size;
-    GST_DEBUG_OBJECT (xvimagesink, "XShm image size is %d", xvimage->size);
+    GST_LOG_OBJECT (xvimagesink, "XShm image size is %d", xvimage->size);
 
     xvimage->SHMInfo.shmid = shmget (IPC_PRIVATE, xvimage->size,
         IPC_CREAT | 0777);
@@ -229,7 +230,7 @@ gst_xvimagesink_xvimage_new (GstXvImageSink * xvimagesink,
       goto beach;
     }
 
-    xvimage->SHMInfo.shmaddr = shmat (xvimage->SHMInfo.shmid, 0, 0);
+    xvimage->SHMInfo.shmaddr = shmat (xvimage->SHMInfo.shmid, NULL, 0);
     if (xvimage->SHMInfo.shmaddr == ((void *) -1)) {
       GST_ELEMENT_ERROR (xvimagesink, RESOURCE, WRITE, (NULL),
           ("Failed to shmat: %s", g_strerror (errno)));
@@ -298,7 +299,7 @@ gst_xvimagesink_xvimage_destroy (GstXvImageSink * xvimagesink,
       shmdt (xvimage->SHMInfo.shmaddr);
     }
     if (xvimage->SHMInfo.shmid > 0)
-      shmctl (xvimage->SHMInfo.shmid, IPC_RMID, 0);
+      shmctl (xvimage->SHMInfo.shmid, IPC_RMID, NULL);
     if (xvimage->xvimage)
       XFree (xvimage->xvimage);
   } else
@@ -325,6 +326,7 @@ gst_xvimagesink_xvimage_put (GstXvImageSink * xvimagesink, GstXvImage * xvimage)
 {
   g_return_if_fail (xvimage != NULL);
   g_return_if_fail (GST_IS_XVIMAGESINK (xvimagesink));
+  g_return_if_fail (xvimagesink->xwindow != NULL);
 
   /* Store a reference to the last image we put */
   if (xvimagesink->cur_image != xvimage)
@@ -358,6 +360,7 @@ gst_xvimagesink_xvimage_put (GstXvImageSink * xvimagesink, GstXvImage * xvimage)
 
   XSync (xvimagesink->xcontext->disp, FALSE);
 
+  xvimagesink->need_expose = FALSE;
   g_mutex_unlock (xvimagesink->x_lock);
 }
 
@@ -407,6 +410,7 @@ gst_xvimagesink_xwindow_new (GstXvImageSink * xvimagesink,
   XGCValues values;
 
   g_return_val_if_fail (GST_IS_XVIMAGESINK (xvimagesink), NULL);
+  GST_LOG_OBJECT (xvimagesink, "new xwindow of %dx%d", width, height);
 
   xwindow = g_new0 (GstXWindow, 1);
 
@@ -477,6 +481,7 @@ gst_xvimagesink_xwindow_resize (GstXvImageSink * xvimagesink,
 
   g_mutex_lock (xvimagesink->x_lock);
 
+  GST_LOG_OBJECT (xvimagesink, "xwindow resize to %dx%d", width, height);
   xwindow->width = width;
   xwindow->height = height;
 
@@ -497,6 +502,9 @@ gst_xvimagesink_xwindow_clear (GstXvImageSink * xvimagesink,
 
   g_mutex_lock (xvimagesink->x_lock);
 
+  XvStopVideo (xvimagesink->xcontext->disp, xvimagesink->xcontext->xv_port_id,
+      xwindow->win);
+
   XSetForeground (xvimagesink->xcontext->disp, xwindow->gc,
       xvimagesink->xcontext->black);
 
@@ -606,7 +614,7 @@ gst_xvimagesink_handle_xevents (GstXvImageSink * xvimagesink, GstPad * pad)
     GST_DEBUG ("xvimagesink pointer moved over window at %d,%d",
         pointer_x, pointer_y);
     gst_navigation_send_mouse_event (GST_NAVIGATION (xvimagesink),
-        "mouse-move", 0, pointer_x, pointer_y);
+        "mouse-move", 0, e.xbutton.x, e.xbutton.y);
   }
 
   /* We get all events on our window to throw them upstream */
@@ -680,16 +688,16 @@ static GstCaps *
 gst_xvimagesink_get_xv_support (GstXvImageSink * xvimagesink,
     GstXContext * xcontext)
 {
-  gint i, nb_adaptors;
+  guint i, nb_adaptors;
   XvAdaptorInfo *adaptors;
-  gint nb_formats;
   XvImageFormatValues *formats = NULL;
   GstCaps *caps = NULL;
+  gint dummy, nb_formats;
 
   g_return_val_if_fail (xcontext != NULL, NULL);
 
   /* First let's check that XVideo extension is available */
-  if (!XQueryExtension (xcontext->disp, "XVideo", &i, &i, &i)) {
+  if (!XQueryExtension (xcontext->disp, "XVideo", &dummy, &dummy, &dummy)) {
     GST_ELEMENT_ERROR (xvimagesink, RESOURCE, SETTINGS, (NULL),
         ("XVideo extension is not available"));
     return NULL;
@@ -767,8 +775,26 @@ gst_xvimagesink_get_xv_support (GstXvImageSink * xvimagesink,
     switch (formats[i].type) {
       case XvRGB:
       {
+        /* our caps system handles 24/32bpp RGB as big-endian. */
+        gint endianness = xcontext->endianness, red_mask = formats[i].red_mask;
+
+        if ((xcontext->bpp == 24 || xcontext->bpp == 32) &&
+            endianness == G_LITTLE_ENDIAN) {
+          endianness = G_BIG_ENDIAN;
+          /* so for some unknown reason (beats me), the red and blue masks
+           * *are* actually in big-endian order, but in little endian format.
+           * Don't ask. I've learned not to. */
+          formats[i].red_mask = GUINT32_TO_BE (formats[i].blue_mask);
+          formats[i].green_mask = GUINT32_TO_BE (formats[i].green_mask);
+          formats[i].blue_mask = GUINT32_TO_BE (red_mask);
+          if (xcontext->bpp == 24) {
+            formats[i].red_mask >>= 8;
+            formats[i].green_mask >>= 8;
+            formats[i].blue_mask >>= 8;
+          }
+        }
         format_caps = gst_caps_new_simple ("video/x-raw-rgb",
-            "endianness", G_TYPE_INT, xcontext->endianness,
+            "endianness", G_TYPE_INT, endianness,
             "depth", G_TYPE_INT, xcontext->depth,
             "bpp", G_TYPE_INT, xcontext->bpp,
             "blue_mask", G_TYPE_INT, formats[i].red_mask,
@@ -954,20 +980,6 @@ gst_xvimagesink_xcontext_get (GstXvImageSink * xvimagesink)
       (ImageByteOrder (xcontext->disp) ==
       LSBFirst) ? G_LITTLE_ENDIAN : G_BIG_ENDIAN;
 
-  /* our caps system handles 24/32bpp RGB as big-endian. */
-  if ((xcontext->bpp == 24 || xcontext->bpp == 32) &&
-      xcontext->endianness == G_LITTLE_ENDIAN) {
-    xcontext->endianness = G_BIG_ENDIAN;
-    xcontext->visual->red_mask = GUINT32_TO_BE (xcontext->visual->red_mask);
-    xcontext->visual->green_mask = GUINT32_TO_BE (xcontext->visual->green_mask);
-    xcontext->visual->blue_mask = GUINT32_TO_BE (xcontext->visual->blue_mask);
-    if (xcontext->bpp == 24) {
-      xcontext->visual->red_mask >>= 8;
-      xcontext->visual->green_mask >>= 8;
-      xcontext->visual->blue_mask >>= 8;
-    }
-  }
-
   xcontext->caps = gst_xvimagesink_get_xv_support (xvimagesink, xcontext);
 
   if (!xcontext->caps) {
@@ -1188,8 +1200,7 @@ gst_xvimagesink_sink_link (GstPad * pad, const GstCaps * caps)
 {
   GstXvImageSink *xvimagesink;
   GstStructure *structure;
-  gint im_format = 0;
-  gboolean ret;
+  guint32 im_format = 0;
   gint video_width, video_height;
   gint video_par_n, video_par_d;        /* video's PAR */
   gint display_par_n, display_par_d;    /* display's PAR */
@@ -1199,17 +1210,22 @@ gst_xvimagesink_sink_link (GstPad * pad, const GstCaps * caps)
 
   xvimagesink = GST_XVIMAGESINK (gst_pad_get_parent (pad));
 
+  if (!xvimagesink->xcontext)
+    return GST_PAD_LINK_DELAYED;
+
   GST_DEBUG_OBJECT (xvimagesink,
       "sinkconnect possible caps %" GST_PTR_FORMAT " with given caps %"
       GST_PTR_FORMAT, xvimagesink->xcontext->caps, caps);
 
   structure = gst_caps_get_structure (caps, 0);
-  ret = gst_structure_get_int (structure, "width", &video_width);
-  ret &= gst_structure_get_int (structure, "height", &video_height);
-  ret &= gst_structure_get_double (structure, "framerate",
-      &xvimagesink->framerate);
-  if (!ret)
+  if (!gst_structure_get_int (structure, "width", &video_width)
+      || !gst_structure_get_int (structure, "height", &video_height)
+      || !gst_structure_get_double (structure, "framerate",
+          &xvimagesink->framerate)) {
     return GST_PAD_LINK_REFUSED;
+  }
+
+  g_mutex_lock (xvimagesink->stream_lock);
 
   xvimagesink->video_width = video_width;
   xvimagesink->video_height = video_height;
@@ -1219,6 +1235,7 @@ gst_xvimagesink_sink_link (GstPad * pad, const GstCaps * caps)
         gst_caps_copy (caps));
   }
   if (im_format == 0) {
+    g_mutex_unlock (xvimagesink->stream_lock);
     return GST_PAD_LINK_REFUSED;
   }
 
@@ -1308,6 +1325,8 @@ gst_xvimagesink_sink_link (GstPad * pad, const GstCaps * caps)
 
   xvimagesink->xcontext->im_format = im_format;
 
+  g_mutex_unlock (xvimagesink->stream_lock);
+
   gst_x_overlay_got_desired_size (GST_X_OVERLAY (xvimagesink),
       GST_VIDEOSINK_WIDTH (xvimagesink), GST_VIDEOSINK_HEIGHT (xvimagesink));
 
@@ -1336,12 +1355,12 @@ gst_xvimagesink_change_state (GstElement * element)
       /* call XSynchronize with the current value of synchronous */
       GST_DEBUG_OBJECT (xvimagesink, "XSynchronize called with %s",
           xvimagesink->synchronous ? "TRUE" : "FALSE");
+      g_mutex_lock (xvimagesink->x_lock);
       XSynchronize (xvimagesink->xcontext->disp, xvimagesink->synchronous);
+      g_mutex_unlock (xvimagesink->x_lock);
       gst_xvimagesink_update_colorbalance (xvimagesink);
       break;
     case GST_STATE_READY_TO_PAUSED:
-      if (xvimagesink->xwindow)
-        gst_xvimagesink_xwindow_clear (xvimagesink, xvimagesink->xwindow);
       xvimagesink->time = 0;
       break;
     case GST_STATE_PAUSED_TO_PLAYING:
@@ -1349,11 +1368,19 @@ gst_xvimagesink_change_state (GstElement * element)
     case GST_STATE_PLAYING_TO_PAUSED:
       break;
     case GST_STATE_PAUSED_TO_READY:
+      g_mutex_lock (xvimagesink->stream_lock);
+      if (xvimagesink->xwindow)
+        gst_xvimagesink_xwindow_clear (xvimagesink, xvimagesink->xwindow);
       xvimagesink->framerate = 0;
       GST_VIDEOSINK_WIDTH (xvimagesink) = 0;
       GST_VIDEOSINK_HEIGHT (xvimagesink) = 0;
+      g_mutex_unlock (xvimagesink->stream_lock);
       break;
     case GST_STATE_READY_TO_NULL:
+      /* We are cleaning our resources here, yes i know chain is not running
+         but the interface can be called to set a window from a different thread
+         and that would crash */
+      g_mutex_lock (xvimagesink->stream_lock);
       if (xvimagesink->xvimage) {
         gst_xvimagesink_xvimage_destroy (xvimagesink, xvimagesink->xvimage);
         xvimagesink->xvimage = NULL;
@@ -1371,6 +1398,7 @@ gst_xvimagesink_change_state (GstElement * element)
         gst_xvimagesink_xcontext_clear (xvimagesink);
         xvimagesink->xcontext = NULL;
       }
+      g_mutex_unlock (xvimagesink->stream_lock);
       break;
   }
 
@@ -1380,6 +1408,59 @@ gst_xvimagesink_change_state (GstElement * element)
   return GST_STATE_SUCCESS;
 }
 
+static void
+gst_xvimagesink_handle_event (GstXvImageSink * xvimagesink, GstEvent * event,
+    GstPad * pad)
+{
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_FILLER:
+    {
+      GstClockTime ts = GST_EVENT_TIMESTAMP (event);
+      GstClockTime dur = gst_event_filler_get_duration (event);
+      GstClockTimeDiff step;
+
+      if (!GST_CLOCK_TIME_IS_VALID (ts)) {
+        ts = xvimagesink->time;
+      }
+      if (GST_CLOCK_TIME_IS_VALID (dur)) {
+        ts += dur;
+      }
+
+      gst_event_unref (event);
+      event = NULL;
+
+      if (xvimagesink->framerate > 0)
+        step = GST_SECOND / xvimagesink->framerate;
+      else
+        step = GST_CLOCK_DIFF (ts, xvimagesink->time);
+
+      /*
+       * Advance the clock to the timestamp of the event, in steps
+       * at the framerate so we send navigation events and redraw nicely.
+       */
+      if (step > 0 && GST_VIDEOSINK_CLOCK (xvimagesink)) {
+        while (xvimagesink->time < ts) {
+          GstClockTime next_ts = MIN (xvimagesink->time + step, ts);
+
+          if (next_ts > xvimagesink->time) {
+            gst_element_wait (GST_ELEMENT (xvimagesink), next_ts);
+          }
+          xvimagesink->time = next_ts;
+
+          g_mutex_lock (xvimagesink->stream_lock);
+          gst_xvimagesink_handle_xevents (xvimagesink, pad);
+          g_mutex_unlock (xvimagesink->stream_lock);
+          gst_xvimagesink_send_pending_navigation (xvimagesink);
+        }
+      }
+
+      break;
+    }
+    default:
+      gst_pad_event_default (pad, event);
+  }
+}
+
 static void
 gst_xvimagesink_chain (GstPad * pad, GstData * data)
 {
@@ -1388,11 +1469,10 @@ gst_xvimagesink_chain (GstPad * pad, GstData * data)
 
   g_return_if_fail (GST_IS_PAD (pad));
   g_return_if_fail (data != NULL);
-
   xvimagesink = GST_XVIMAGESINK (gst_pad_get_parent (pad));
-
+  g_return_if_fail (xvimagesink != NULL);
   if (GST_IS_EVENT (data)) {
-    gst_pad_event_default (pad, GST_EVENT (data));
+    gst_xvimagesink_handle_event (xvimagesink, GST_EVENT (data), pad);
     return;
   }
 
@@ -1403,11 +1483,12 @@ gst_xvimagesink_chain (GstPad * pad, GstData * data)
   }
   GST_LOG_OBJECT (xvimagesink, "clock wait: %" GST_TIME_FORMAT,
       GST_TIME_ARGS (xvimagesink->time));
-
   if (GST_VIDEOSINK_CLOCK (xvimagesink)) {
     gst_element_wait (GST_ELEMENT (xvimagesink), xvimagesink->time);
   }
 
+  g_mutex_lock (xvimagesink->stream_lock);
+
   /* If this buffer has been allocated using our buffer management we simply
      put the ximage which is in the PRIVATE pointer */
   if (GST_BUFFER_FREE_DATA_FUNC (buf) == gst_xvimagesink_buffer_free) {
@@ -1424,6 +1505,7 @@ gst_xvimagesink_chain (GstPad * pad, GstData * data)
         gst_buffer_unref (buf);
         GST_ELEMENT_ERROR (xvimagesink, CORE, NEGOTIATION, (NULL),
             ("Failed creating an XvImage in xvimagesink chain function."));
+        g_mutex_unlock (xvimagesink->stream_lock);
         return;
       }
     }
@@ -1440,8 +1522,10 @@ gst_xvimagesink_chain (GstPad * pad, GstData * data)
   }
 
   gst_buffer_unref (buf);
-
   gst_xvimagesink_handle_xevents (xvimagesink, pad);
+
+  g_mutex_unlock (xvimagesink->stream_lock);
+  gst_xvimagesink_send_pending_navigation (xvimagesink);
 }
 
 /* Buffer management */
@@ -1453,13 +1537,11 @@ gst_xvimagesink_buffer_free (GstBuffer * buffer)
   GstXvImage *xvimage;
 
   xvimage = GST_BUFFER_PRIVATE (buffer);
-
   g_assert (GST_IS_XVIMAGESINK (xvimage->xvimagesink));
   xvimagesink = xvimage->xvimagesink;
-
   /* If our geometry changed we can't reuse that image. */
-  if ((xvimage->width != GST_VIDEOSINK_WIDTH (xvimagesink)) ||
-      (xvimage->height != GST_VIDEOSINK_HEIGHT (xvimagesink)))
+  if ((xvimage->width != xvimagesink->video_width) ||
+      (xvimage->height != xvimagesink->video_height))
     gst_xvimagesink_xvimage_destroy (xvimagesink, xvimage);
   else {
     /* In that case we can reuse the image and add it to our image pool. */
@@ -1479,22 +1561,19 @@ gst_xvimagesink_buffer_alloc (GstPad * pad, guint64 offset, guint size)
   gboolean not_found = TRUE;
 
   xvimagesink = GST_XVIMAGESINK (gst_pad_get_parent (pad));
-
   g_mutex_lock (xvimagesink->pool_lock);
-
   /* Walking through the pool cleaning unusable images and searching for a
      suitable one */
   while (not_found && xvimagesink->image_pool) {
     xvimage = xvimagesink->image_pool->data;
-
     if (xvimage) {
       /* Removing from the pool */
-      xvimagesink->image_pool = g_slist_delete_link (xvimagesink->image_pool,
+      xvimagesink->image_pool =
+          g_slist_delete_link (xvimagesink->image_pool,
           xvimagesink->image_pool);
-
       /* We check for geometry or image format changes */
-      if ((xvimage->width != GST_VIDEOSINK_WIDTH (xvimagesink)) ||
-          (xvimage->height != GST_VIDEOSINK_HEIGHT (xvimagesink)) ||
+      if ((xvimage->width != xvimagesink->video_width) ||
+          (xvimage->height != xvimagesink->video_height) ||
           (xvimage->im_format != xvimagesink->xcontext->im_format)) {
         /* This image is unusable. Destroying... */
         gst_xvimagesink_xvimage_destroy (xvimagesink, xvimage);
@@ -1507,21 +1586,19 @@ gst_xvimagesink_buffer_alloc (GstPad * pad, guint64 offset, guint size)
   }
 
   g_mutex_unlock (xvimagesink->pool_lock);
-
   if (!xvimage) {
     /* We found no suitable image in the pool. Creating... */
-    GST_DEBUG_OBJECT (xvimagesink, "no usable image in pool, creating xvimage");
-    xvimage = gst_xvimagesink_xvimage_new (xvimagesink,
-        xvimagesink->video_width, xvimagesink->video_height);
+    GST_LOG_OBJECT (xvimagesink, "no usable image in pool, creating xvimage");
+    xvimage =
+        gst_xvimagesink_xvimage_new (xvimagesink, xvimagesink->video_width,
+        xvimagesink->video_height);
   }
 
   if (xvimage) {
     buffer = gst_buffer_new ();
-
     /* Storing some pointers in the buffer */
     GST_BUFFER_PRIVATE (buffer) = xvimage;
-
-    GST_BUFFER_DATA (buffer) = xvimage->xvimage->data;
+    GST_BUFFER_DATA (buffer) = (guint8 *) xvimage->xvimage->data;
     GST_BUFFER_FREE_DATA_FUNC (buffer) = gst_xvimagesink_buffer_free;
     GST_BUFFER_SIZE (buffer) = xvimage->size;
     return buffer;
@@ -1534,8 +1611,8 @@ gst_xvimagesink_buffer_alloc (GstPad * pad, guint64 offset, guint size)
 static gboolean
 gst_xvimagesink_interface_supported (GstImplementsInterface * iface, GType type)
 {
-  g_assert (type == GST_TYPE_NAVIGATION ||
-      type == GST_TYPE_X_OVERLAY || type == GST_TYPE_COLOR_BALANCE);
+  g_assert (type == GST_TYPE_NAVIGATION || type == GST_TYPE_X_OVERLAY
+      || type == GST_TYPE_COLOR_BALANCE);
   return TRUE;
 }
 
@@ -1545,31 +1622,73 @@ gst_xvimagesink_interface_init (GstImplementsInterfaceClass * klass)
   klass->supported = gst_xvimagesink_interface_supported;
 }
 
+/*
+ * This function is called with the stream-lock held
+ */
+static void
+gst_xvimagesink_send_pending_navigation (GstXvImageSink * xvimagesink)
+{
+  GSList *cur;
+  GSList *pend_events;
+
+  g_mutex_lock (xvimagesink->nav_lock);
+  pend_events = xvimagesink->pend_nav_events;
+  xvimagesink->pend_nav_events = NULL;
+  g_mutex_unlock (xvimagesink->nav_lock);
+
+  cur = pend_events;
+  while (cur) {
+    GstEvent *event = cur->data;
+    GstStructure *structure;
+    double x, y;
+
+    if (event) {
+      structure = event->event_data.structure.structure;
+      if (!GST_PAD_PEER (GST_VIDEOSINK_PAD (xvimagesink))) {
+        gst_event_unref (event);
+        cur = g_slist_next (cur);
+        continue;
+      }
+
+      /* Converting pointer coordinates to the non scaled geometry */
+      g_mutex_lock (xvimagesink->stream_lock);
+      if (gst_structure_get_double (structure, "pointer_x", &x)) {
+        x *= xvimagesink->video_width;
+        x /= xvimagesink->xwindow->width;
+        gst_structure_set (structure, "pointer_x", G_TYPE_DOUBLE, x, NULL);
+      }
+      if (gst_structure_get_double (structure, "pointer_y", &y)) {
+        y *= xvimagesink->video_height;
+        y /= xvimagesink->xwindow->height;
+        gst_structure_set (structure, "pointer_y", G_TYPE_DOUBLE, y, NULL);
+      }
+      g_mutex_unlock (xvimagesink->stream_lock);
+
+      GST_DATA_FLAG_UNSET (event, GST_EVENT_COMMON_FLAG_NEED_RESPONSE);
+
+      gst_pad_send_event (gst_pad_get_peer (GST_VIDEOSINK_PAD
+              (xvimagesink)), event);
+    }
+    cur = g_slist_next (cur);
+  }
+
+  g_slist_free (pend_events);
+}
+
 static void
 gst_xvimagesink_navigation_send_event (GstNavigation * navigation,
     GstStructure * structure)
 {
   GstXvImageSink *xvimagesink = GST_XVIMAGESINK (navigation);
   GstEvent *event;
-  double x, y;
 
   event = gst_event_new (GST_EVENT_NAVIGATION);
+
   event->event_data.structure.structure = structure;
-
-  /* Converting pointer coordinates to the non scaled geometry */
-  if (gst_structure_get_double (structure, "pointer_x", &x)) {
-    x *= GST_VIDEOSINK_WIDTH (xvimagesink);
-    x /= xvimagesink->xwindow->width;
-    gst_structure_set (structure, "pointer_x", G_TYPE_DOUBLE, x, NULL);
-  }
-  if (gst_structure_get_double (structure, "pointer_y", &y)) {
-    y *= GST_VIDEOSINK_HEIGHT (xvimagesink);
-    y /= xvimagesink->xwindow->height;
-    gst_structure_set (structure, "pointer_y", G_TYPE_DOUBLE, y, NULL);
-  }
-
-  gst_pad_send_event (gst_pad_get_peer (GST_VIDEOSINK_PAD (xvimagesink)),
-      event);
+  g_mutex_lock (xvimagesink->nav_lock);
+  xvimagesink->pend_nav_events =
+      g_slist_prepend (xvimagesink->pend_nav_events, event);
+  g_mutex_unlock (xvimagesink->nav_lock);
 }
 
 static void
@@ -1586,22 +1705,21 @@ gst_xvimagesink_set_xwindow_id (GstXOverlay * overlay, XID xwindow_id)
   XWindowAttributes attr;
 
   g_return_if_fail (GST_IS_XVIMAGESINK (xvimagesink));
-
   /* If we already use that window return */
   if (xvimagesink->xwindow && (xwindow_id == xvimagesink->xwindow->win))
     return;
-
   /* If the element has not initialized the X11 context try to do so */
   if (!xvimagesink->xcontext &&
       !(xvimagesink->xcontext = gst_xvimagesink_xcontext_get (xvimagesink)))
     /* we have thrown a GST_ELEMENT_ERROR now */
     return;
-
   gst_xvimagesink_update_colorbalance (xvimagesink);
-
+  /* We acquire the stream lock while setting this window in the element.
+     We are basically cleaning tons of stuff replacing the old window, putting
+     images while we do that would surely crash */
+  g_mutex_lock (xvimagesink->stream_lock);
   /* Clear image pool as the images are unusable anyway */
   gst_xvimagesink_imagepool_clear (xvimagesink);
-
   /* Clear the xvimage */
   if (xvimagesink->xvimage) {
     gst_xvimagesink_xvimage_destroy (xvimagesink, xvimagesink->xvimage);
@@ -1618,16 +1736,16 @@ gst_xvimagesink_set_xwindow_id (GstXOverlay * overlay, XID xwindow_id)
   if (xwindow_id == 0) {
     /* If no width/height caps nego did not happen window will be created
        during caps nego then */
-    if (GST_VIDEOSINK_WIDTH (xvimagesink) && GST_VIDEOSINK_HEIGHT (xvimagesink)) {
-      xwindow = gst_xvimagesink_xwindow_new (xvimagesink,
+    if (GST_VIDEOSINK_WIDTH (xvimagesink)
+        && GST_VIDEOSINK_HEIGHT (xvimagesink)) {
+      xwindow =
+          gst_xvimagesink_xwindow_new (xvimagesink,
           GST_VIDEOSINK_WIDTH (xvimagesink),
           GST_VIDEOSINK_HEIGHT (xvimagesink));
     }
   } else {
     xwindow = g_new0 (GstXWindow, 1);
-
     xwindow->win = xwindow_id;
-
     /* We get window geometry, set the event we want to receive,
        and create a GC */
     g_mutex_lock (xvimagesink->x_lock);
@@ -1635,6 +1753,8 @@ gst_xvimagesink_set_xwindow_id (GstXOverlay * overlay, XID xwindow_id)
     xwindow->width = attr.width;
     xwindow->height = attr.height;
     xwindow->internal = FALSE;
+    GST_LOG_OBJECT (xvimagesink, "xwindow with xid %ld is %dx%d", xwindow_id,
+        xwindow->width, xwindow->height);
     XSelectInput (xvimagesink->xcontext->disp, xwindow->win, ExposureMask |
         StructureNotifyMask | PointerMotionMask | KeyPressMask |
         KeyReleaseMask);
@@ -1646,6 +1766,7 @@ gst_xvimagesink_set_xwindow_id (GstXOverlay * overlay, XID xwindow_id)
 
   if (xwindow)
     xvimagesink->xwindow = xwindow;
+  g_mutex_unlock (xvimagesink->stream_lock);
 }
 
 static void
@@ -1667,18 +1788,31 @@ gst_xvimagesink_expose (GstXOverlay * overlay)
   if (!xvimagesink->xwindow)
     return;
 
+  /* Set the need_expose flag before we wait for the lock. 
+   * If it's clear by the time we get to put the image,
+   * the stream beat us to it so don't bother
+   */
+  xvimagesink->need_expose = TRUE;
+
+  /* We don't want chain to iterate while we do that. We might act on random
+     cur_image and different geometry */
+  g_mutex_lock (xvimagesink->stream_lock);
+
   /* Update the window geometry */
   g_mutex_lock (xvimagesink->x_lock);
   XGetWindowAttributes (xvimagesink->xcontext->disp,
       xvimagesink->xwindow->win, &attr);
   g_mutex_unlock (xvimagesink->x_lock);
-
   xvimagesink->xwindow->width = attr.width;
   xvimagesink->xwindow->height = attr.height;
+  if (xvimagesink->need_expose && xvimagesink->cur_image) {
+    GST_LOG_OBJECT (xvimagesink, "xwindow exposed is %dx%d", attr.width,
+        attr.height);
 
-  if (xvimagesink->cur_image) {
     gst_xvimagesink_xvimage_put (xvimagesink, xvimagesink->cur_image);
   }
+
+  g_mutex_unlock (xvimagesink->stream_lock);
 }
 
 static void
@@ -1695,7 +1829,6 @@ gst_xvimagesink_colorbalance_list_channels (GstColorBalance * balance)
   GstXvImageSink *xvimagesink = GST_XVIMAGESINK (balance);
 
   g_return_val_if_fail (GST_IS_XVIMAGESINK (xvimagesink), NULL);
-
   if (xvimagesink->xcontext)
     return xvimagesink->xcontext->channels_list;
   else
@@ -1710,13 +1843,10 @@ gst_xvimagesink_colorbalance_set_value (GstColorBalance * balance,
 
   g_return_if_fail (GST_IS_XVIMAGESINK (xvimagesink));
   g_return_if_fail (channel->label != NULL);
-
   xvimagesink->cb_changed = TRUE;
-
   /* Normalize val to [-1000, 1000] */
   value = -1000 + 2000 * (value - channel->min_value) /
       (channel->max_value - channel->min_value);
-
   if (g_ascii_strcasecmp (channel->label, "XV_HUE") == 0) {
     xvimagesink->hue = value;
   } else if (g_ascii_strcasecmp (channel->label, "XV_SATURATION") == 0) {
@@ -1742,7 +1872,6 @@ gst_xvimagesink_colorbalance_get_value (GstColorBalance * balance,
 
   g_return_val_if_fail (GST_IS_XVIMAGESINK (xvimagesink), 0);
   g_return_val_if_fail (channel->label != NULL, 0);
-
   if (g_ascii_strcasecmp (channel->label, "XV_HUE") == 0) {
     value = xvimagesink->hue;
   } else if (g_ascii_strcasecmp (channel->label, "XV_SATURATION") == 0) {
@@ -1758,7 +1887,6 @@ gst_xvimagesink_colorbalance_get_value (GstColorBalance * balance,
   /* Normalize val to [channel->min_value, channel->max_value] */
   value = channel->min_value + (channel->max_value - channel->min_value) *
       (value + 1000) / 2000;
-
   return value;
 }
 
@@ -1784,9 +1912,7 @@ gst_xvimagesink_set_property (GObject * object, guint prop_id,
   GstXvImageSink *xvimagesink;
 
   g_return_if_fail (GST_IS_XVIMAGESINK (object));
-
   xvimagesink = GST_XVIMAGESINK (object);
-
   switch (prop_id) {
     case ARG_HUE:
       xvimagesink->hue = g_value_get_int (value);
@@ -1814,9 +1940,11 @@ gst_xvimagesink_set_property (GObject * object, guint prop_id,
     case ARG_SYNCHRONOUS:
       xvimagesink->synchronous = g_value_get_boolean (value);
       if (xvimagesink->xcontext) {
+        g_mutex_lock (xvimagesink->x_lock);
         XSynchronize (xvimagesink->xcontext->disp, xvimagesink->synchronous);
         GST_DEBUG_OBJECT (xvimagesink, "XSynchronize called with %s",
             xvimagesink->synchronous ? "TRUE" : "FALSE");
+        g_mutex_unlock (xvimagesink->x_lock);
       }
       break;
     case ARG_PIXEL_ASPECT_RATIO:
@@ -1844,9 +1972,7 @@ gst_xvimagesink_get_property (GObject * object, guint prop_id,
   GstXvImageSink *xvimagesink;
 
   g_return_if_fail (GST_IS_XVIMAGESINK (object));
-
   xvimagesink = GST_XVIMAGESINK (object);
-
   switch (prop_id) {
     case ARG_HUE:
       g_value_set_int (value, xvimagesink->hue);
@@ -1885,7 +2011,6 @@ gst_xvimagesink_finalize (GObject * object)
   GstXvImageSink *xvimagesink;
 
   xvimagesink = GST_XVIMAGESINK (object);
-
   if (xvimagesink->display_name) {
     g_free (xvimagesink->display_name);
     xvimagesink->display_name = NULL;
@@ -1899,10 +2024,27 @@ gst_xvimagesink_finalize (GObject * object)
     g_mutex_free (xvimagesink->x_lock);
     xvimagesink->x_lock = NULL;
   }
+  if (xvimagesink->stream_lock) {
+    g_mutex_free (xvimagesink->stream_lock);
+    xvimagesink->stream_lock = NULL;
+  }
   if (xvimagesink->pool_lock) {
     g_mutex_free (xvimagesink->pool_lock);
     xvimagesink->pool_lock = NULL;
   }
+  if (xvimagesink->nav_lock) {
+    g_mutex_free (xvimagesink->nav_lock);
+    xvimagesink->nav_lock = NULL;
+  }
+
+  while (xvimagesink->pend_nav_events) {
+    GstEvent *event = xvimagesink->pend_nav_events->data;
+
+    xvimagesink->pend_nav_events =
+        g_slist_delete_link (xvimagesink->pend_nav_events,
+        xvimagesink->pend_nav_events);
+    gst_event_unref (event);
+  }
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -1913,10 +2055,8 @@ gst_xvimagesink_init (GstXvImageSink * xvimagesink)
   GST_VIDEOSINK_PAD (xvimagesink) =
       gst_pad_new_from_template (gst_static_pad_template_get
       (&gst_xvimagesink_sink_template_factory), "sink");
-
   gst_element_add_pad (GST_ELEMENT (xvimagesink),
       GST_VIDEOSINK_PAD (xvimagesink));
-
   gst_pad_set_chain_function (GST_VIDEOSINK_PAD (xvimagesink),
       gst_xvimagesink_chain);
   gst_pad_set_link_function (GST_VIDEOSINK_PAD (xvimagesink),
@@ -1927,31 +2067,28 @@ gst_xvimagesink_init (GstXvImageSink * xvimagesink)
       gst_xvimagesink_fixate);
   gst_pad_set_bufferalloc_function (GST_VIDEOSINK_PAD (xvimagesink),
       gst_xvimagesink_buffer_alloc);
-
   xvimagesink->display_name = NULL;
   xvimagesink->xcontext = NULL;
   xvimagesink->xwindow = NULL;
   xvimagesink->xvimage = NULL;
   xvimagesink->cur_image = NULL;
-
   xvimagesink->hue = xvimagesink->saturation = 0;
   xvimagesink->contrast = xvimagesink->brightness = 0;
   xvimagesink->cb_changed = FALSE;
-
   xvimagesink->framerate = 0;
   xvimagesink->video_width = 0;
   xvimagesink->video_height = 0;
-
   xvimagesink->x_lock = g_mutex_new ();
-
+  xvimagesink->stream_lock = g_mutex_new ();
   xvimagesink->image_pool = NULL;
   xvimagesink->pool_lock = g_mutex_new ();
-
   xvimagesink->synchronous = FALSE;
+  xvimagesink->need_expose = FALSE;
   xvimagesink->par = NULL;
-
   GST_FLAG_SET (xvimagesink, GST_ELEMENT_THREAD_SUGGESTED);
   GST_FLAG_SET (xvimagesink, GST_ELEMENT_EVENT_AWARE);
+  xvimagesink->nav_lock = g_mutex_new ();
+  xvimagesink->pend_nav_events = NULL;
 }
 
 static void
@@ -1960,7 +2097,6 @@ gst_xvimagesink_base_init (gpointer g_class)
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
   gst_element_class_set_details (element_class, &gst_xvimagesink_details);
-
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_xvimagesink_sink_template_factory));
 }
@@ -1973,18 +2109,16 @@ gst_xvimagesink_class_init (GstXvImageSinkClass * klass)
 
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
-
   parent_class = g_type_class_ref (GST_TYPE_VIDEOSINK);
-
   g_object_class_install_property (gobject_class, ARG_CONTRAST,
-      g_param_spec_int ("contrast", "Contrast", "The contrast of the video",
-          -1000, 1000, 0, G_PARAM_READWRITE));
+      g_param_spec_int ("contrast", "Contrast",
+          "The contrast of the video", -1000, 1000, 0, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_BRIGHTNESS,
       g_param_spec_int ("brightness", "Brightness",
           "The brightness of the video", -1000, 1000, 0, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_HUE,
-      g_param_spec_int ("hue", "Hue", "The hue of the video", -1000, 1000, 0,
-          G_PARAM_READWRITE));
+      g_param_spec_int ("hue", "Hue", "The hue of the video", -1000, 1000,
+          0, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_SATURATION,
       g_param_spec_int ("saturation", "Saturation",
           "The saturation of the video", -1000, 1000, 0, G_PARAM_READWRITE));
@@ -1994,16 +2128,14 @@ gst_xvimagesink_class_init (GstXvImageSinkClass * klass)
   g_object_class_install_property (gobject_class, ARG_SYNCHRONOUS,
       g_param_spec_boolean ("synchronous", "Synchronous",
           "When enabled, runs "
-          "the X display in synchronous mode. (used only for debugging)", FALSE,
-          G_PARAM_READWRITE));
+          "the X display in synchronous mode. (used only for debugging)",
+          FALSE, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_PIXEL_ASPECT_RATIO,
       g_param_spec_string ("pixel-aspect-ratio", "Pixel Aspect Ratio",
           "The pixel aspect ratio of the device", "1/1", G_PARAM_READWRITE));
-
   gobject_class->finalize = gst_xvimagesink_finalize;
   gobject_class->set_property = gst_xvimagesink_set_property;
   gobject_class->get_property = gst_xvimagesink_get_property;
-
   gstelement_class->change_state = gst_xvimagesink_change_state;
 }
 
@@ -2038,28 +2170,23 @@ gst_xvimagesink_get_type (void)
     };
     static const GInterfaceInfo iface_info = {
       (GInterfaceInitFunc) gst_xvimagesink_interface_init,
-      NULL,
-      NULL,
+      NULL, NULL,
     };
     static const GInterfaceInfo navigation_info = {
       (GInterfaceInitFunc) gst_xvimagesink_navigation_init,
-      NULL,
-      NULL,
+      NULL, NULL,
     };
     static const GInterfaceInfo overlay_info = {
       (GInterfaceInitFunc) gst_xvimagesink_xoverlay_init,
-      NULL,
-      NULL,
+      NULL, NULL,
     };
     static const GInterfaceInfo colorbalance_info = {
       (GInterfaceInitFunc) gst_xvimagesink_colorbalance_init,
       NULL,
       NULL,
     };
-
     xvimagesink_type = g_type_register_static (GST_TYPE_VIDEOSINK,
         "GstXvImageSink", &xvimagesink_info, 0);
-
     g_type_add_interface_static (xvimagesink_type,
         GST_TYPE_IMPLEMENTS_INTERFACE, &iface_info);
     g_type_add_interface_static (xvimagesink_type, GST_TYPE_NAVIGATION,
@@ -2079,14 +2206,11 @@ plugin_init (GstPlugin * plugin)
   /* Loading the library containing GstVideoSink, our parent object */
   if (!gst_library_load ("gstvideo"))
     return FALSE;
-
   if (!gst_element_register (plugin, "xvimagesink",
           GST_RANK_PRIMARY, GST_TYPE_XVIMAGESINK))
     return FALSE;
-
   GST_DEBUG_CATEGORY_INIT (gst_debug_xvimagesink, "xvimagesink", 0,
       "xvimagesink element");
-
   return TRUE;
 }
 
